Package: bcmixed
Type: Package
Title: Mixed Effect Model with the Box-Cox Transformation
Version: 0.1.6
Authors@R: c(person("Kazushi", "Maruo", email = "kazushi.maruo@gmail.com",
                  role = c("aut", "cre")),
                 person("Ryota", "Ishii", role = "aut"),
                 person("Yusuke", "Yamaguchi", role = "ctb"),
                 person("Masahiko", "Gosho", role = "ctb"))
Description: Inference on the marginal model of the mixed effect model with 
    the Box-Cox transformation and on the model median differences between
    treatment groups for longitudinal randomized clinical trials. These 
    statistical methods are proposed by Maruo et al. (2017) 
    <doi:10.1002/sim.7279>.
Depends: R (>= 3.3.3)
Imports: MASS (>= 7.3-45), nlme (>= 3.1-131)
Suggests: testthat (>= 1.0.2)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-17 15:37:20 UTC; kazus
Author: Kazushi Maruo [aut, cre],
  Ryota Ishii [aut],
  Yusuke Yamaguchi [ctb],
  Masahiko Gosho [ctb]
Maintainer: Kazushi Maruo <kazushi.maruo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-17 16:20:07 UTC
Built: R 4.5.2; ; 2026-05-17 19:03:56 UTC; unix

Package: bcmixed
Type: Package
Title: Mixed Effect Model with the Box-Cox Transformation
Version: 0.1.6
Authors@R: c(person("Kazushi", "Maruo", email = "kazushi.maruo@gmail.com",
                  role = c("aut", "cre")),
                 person("Ryota", "Ishii", role = "aut"),
                 person("Yusuke", "Yamaguchi", role = "ctb"),
                 person("Masahiko", "Gosho", role = "ctb"))
Description: Inference on the marginal model of the mixed effect model with 
    the Box-Cox transformation and on the model median differences between
    treatment groups for longitudinal randomized clinical trials. These 
    statistical methods are proposed by Maruo et al. (2017) 
    <doi:10.1002/sim.7279>.
Depends: R (>= 3.3.3)
Imports: MASS (>= 7.3-45), nlme (>= 3.1-131)
Suggests: testthat (>= 1.0.2)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-17 15:37:20 UTC; kazus
Author: Kazushi Maruo [aut, cre],
  Ryota Ishii [aut],
  Yusuke Yamaguchi [ctb],
  Masahiko Gosho [ctb]
Maintainer: Kazushi Maruo <kazushi.maruo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-17 16:20:07 UTC
Built: R 4.6.0; ; 2026-05-17 20:22:30 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(boxplot,bcmmrm)
S3method(coef,bcmarg)
S3method(coef,bcmmrm)
S3method(fitted,bcmarg)
S3method(fitted,bcmmrm)
S3method(logLik,bcmarg)
S3method(logLik,bcmmrm)
S3method(plot,bcmmrm)
S3method(predict,bcmarg)
S3method(print,bcmarg)
S3method(print,bcmmrm)
S3method(print,summary.bcmarg)
S3method(print,summary.bcmmrm)
S3method(summary,bcmarg)
S3method(summary,bcmmrm)
S3method(vcov,bcmarg)
S3method(vcov,bcmmrm)
export(bcmarg)
export(bcmmrm)
export(bcreg)
export(bct)
export(bct.v)
importFrom(MASS,ginv)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,grid)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(nlme,corAR1)
importFrom(nlme,corCompSymm)
importFrom(nlme,corMatrix)
importFrom(nlme,corSymm)
importFrom(nlme,gls)
importFrom(nlme,glsControl)
importFrom(nlme,varIdent)
importFrom(stats,coef)
importFrom(stats,ftable)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(stats,xtabs)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# bcmixed

The bcmixed package provides two categories of important functions:
bcmarg and bcmmrm. The bcmarg function provides inferences on the
marginal model of the mixed effect model with the Box-Cox transformation
and the bcmmrm function provides inferences on the model median
differences between treatment groups for longitudinal randomized
clinical trials. These statistical methods are proposed by [Maruo et
al. (2017)](https://doi.org/10.1002/sim.7279).

## Installation

You can install the released version of bcmixed from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("bcmixed")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("kzkzmr/bcmixed")
```

## Example

This is a basic example which shows you how to solve a common problem:

``` r
library(bcmixed)
data(aidscd4)
# Marginal model of mixed model with the Box-Cox transformation
res1 <- bcmarg(cd4 ~ as.factor(treatment) * as.factor(weekc) + age,
       data = aidscd4, time = weekc, id = id)
summary(res1)
#> Box-Cox transformed mixed model analysis
#>   Formula: cd4 ~ as.factor(treatment) * as.factor(weekc) + age 
#>   Time: weekc 
#>   ID: id 
#>   Covariance structure: "UN" 
#>   Data: aidscd4 
#>   Log-likelihood: -13623.31
#>   Estimated transformation parameter:  0.183 
#> 
#> Coefficients on the transformed scale:
#>                                            Value Std.Error t-value p-value
#> (Intercept)                               3.1483   0.25917  12.147   0.000
#> as.factor(treatment)2                     0.2967   0.16995   1.746   0.081
#> as.factor(treatment)3                     0.4716   0.16946   2.783   0.005
#> as.factor(treatment)4                     0.7614   0.16763   4.542   0.000
#> as.factor(weekc)16                       -0.2254   0.09248  -2.437   0.015
#> as.factor(weekc)24                       -0.4863   0.09781  -4.972   0.000
#> as.factor(weekc)32                       -0.7246   0.10834  -6.688   0.000
#> age                                       0.0207   0.00612   3.384   0.001
#> as.factor(treatment)2:as.factor(weekc)16 -0.1386   0.12994  -1.067   0.286
#> as.factor(treatment)3:as.factor(weekc)16 -0.0500   0.12943  -0.387   0.699
#> as.factor(treatment)4:as.factor(weekc)16  0.0682   0.12896   0.529   0.597
#> as.factor(treatment)2:as.factor(weekc)24 -0.2078   0.13763  -1.510   0.131
#> as.factor(treatment)3:as.factor(weekc)24 -0.0235   0.13758  -0.171   0.864
#> as.factor(treatment)4:as.factor(weekc)24  0.0821   0.13754   0.597   0.551
#> as.factor(treatment)2:as.factor(weekc)32 -0.1034   0.15508  -0.667   0.505
#> as.factor(treatment)3:as.factor(weekc)32 -0.1018   0.15353  -0.663   0.507
#> as.factor(treatment)4:as.factor(weekc)32  0.1436   0.15162   0.947   0.344
#> 
#> Covariance parameters on the transformed scale:
#> UN(1,1) UN(1,2) UN(1,3) UN(1,4) UN(2,2) UN(2,3) UN(2,4) UN(3,3) UN(3,4) UN(4,4) 
#>    3.72    2.76    2.61    2.34    3.47    2.67    2.56    3.24    2.68    3.27 
#> 
#> Correlations on the transformed scale:
#>        8    16    24    32
#> 8  1.000 0.767 0.751 0.670
#> 16 0.767 1.000 0.797 0.761
#> 24 0.751 0.797 1.000 0.821
#> 32 0.670 0.761 0.821 1.000

# Box-Cox transformation for the baseline
lmd.bl <- bcmarg(cd4.bl ~ 1, data = aidscd4[aidscd4$weekc == 8, ])$lambda
aidscd4$cd4.bl.tr <- bct(aidscd4$cd4.bl, lmd.bl)

# Inference on model median differences between groups at each time point
res2 <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4, time = weekc,
       id = id, covv = c("cd4.bl.tr", "sex"), cfactor = c(0, 1),
       glabel = c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))

# Summarize
print(res2)
#> Model median estimation based on MMRM with Box-Cox transformation
#>   Outcome: cd4 
#>   Group: treatment 
#>   Time: weekc 
#>   ID: id 
#>   Covariate(s): c("cd4.bl.tr", "sex") 
#>   Covariance structure: "UN" 
#>   Data: aidscd4 
#>   Estimated transformation parameter:  0.154 
#>   Log-likelihood: -13322.36
#> 
#> Model median estimates (row: group, col: time):
#>   treatment | weekc    8   16   24   32
#> 1           Zid/Did 18.9 16.5 14.1 12.1
#> 2           Zid+Zal 22.0 17.9 14.6 13.5
#> 3           Zid+Did 24.5 20.9 18.2 15.1
#> 4       Zid+Did+Nev 30.1 27.8 24.2 21.8

summary(res2)
#> Model median inference based on MMRM with Box-Cox transformation
#> 
#> Data and variable information:
#>   Outcome: cd4 
#>   Group: treatment 
#>   Time: weekc 
#>   ID: id 
#>   Covariate(s): c("cd4.bl.tr", "sex") 
#>   Data: aidscd4 
#> 
#> Analysis information:
#>   Covariance structure: "UN" 
#>   Robust inference: TRUE 
#>   Empirical small sample adjustment: TRUE 
#>   Confidence level: 0.95 
#> 
#> Analysis results:
#>   Estimated transformation parameter:  0.154 
#> 
#>  
#> Model median inferences for weekc = 8 
#>  
#>     treatment median    SE lower.CL upper.CL
#> 1     Zid/Did   18.9 0.862     17.2     20.6
#> 2     Zid+Zal   22.0 1.124     19.8     24.2
#> 3     Zid+Did   24.5 1.465     21.6     27.4
#> 4 Zid+Did+Nev   30.1 1.597     27.0     33.3
#> 
#>  
#> Model median inferences for weekc = 16 
#>  
#>     treatment median    SE lower.CL upper.CL
#> 1     Zid/Did   16.5 0.799     15.0     18.1
#> 2     Zid+Zal   17.9 0.932     16.1     19.8
#> 3     Zid+Did   20.9 1.207     18.5     23.2
#> 4 Zid+Did+Nev   27.8 1.596     24.6     30.9
#> 
#>  
#> Model median inferences for weekc = 24 
#>  
#>     treatment median    SE lower.CL upper.CL
#> 1     Zid/Did   14.1 0.716     12.7     15.5
#> 2     Zid+Zal   14.6 0.864     12.9     16.3
#> 3     Zid+Did   18.2 1.175     15.9     20.5
#> 4 Zid+Did+Nev   24.2 1.510     21.3     27.2
#> 
#>  
#> Model median inferences for weekc = 32 
#>  
#>     treatment median    SE lower.CL upper.CL
#> 1     Zid/Did   12.1 0.662     10.8     13.4
#> 2     Zid+Zal   13.5 0.813     11.9     15.1
#> 3     Zid+Did   15.1 1.019     13.1     17.1
#> 4 Zid+Did+Nev   21.8 1.376     19.1     24.5
#> 
#>  
#> Inferences of model median difference between groups ( treatment 1 - treatment 0 ) for weekc = 8 
#>  
#>   treatment 1 treatment 0 delta   SE lower.CL upper.CL t.value p.value
#> 1     Zid+Zal     Zid/Did  3.12 1.40    0.363     5.87    2.22   0.027
#> 2     Zid+Did     Zid/Did  5.64 1.69    2.325     8.96    3.34   0.001
#> 3 Zid+Did+Nev     Zid/Did 11.25 1.80    7.711    14.80    6.24   0.000
#> 4     Zid+Did     Zid+Zal  2.53 1.83   -1.059     6.12    1.39   0.167
#> 5 Zid+Did+Nev     Zid+Zal  8.14 1.93    4.349    11.93    4.22   0.000
#> 6 Zid+Did+Nev     Zid+Did  5.61 2.16    1.372     9.85    2.60   0.010
#> 
#>  
#> Inferences of model median difference between groups ( treatment 1 - treatment 0 ) for weekc = 16 
#>  
#>   treatment 1 treatment 0 delta   SE lower.CL upper.CL t.value p.value
#> 1     Zid+Zal     Zid/Did  1.38 1.21  -1.0015     3.75    1.14   0.256
#> 2     Zid+Did     Zid/Did  4.31 1.43   1.4940     7.12    3.01   0.003
#> 3 Zid+Did+Nev     Zid/Did 11.22 1.78   7.7118    14.73    6.29   0.000
#> 4     Zid+Did     Zid+Zal  2.93 1.50  -0.0162     5.88    1.95   0.051
#> 5 Zid+Did+Nev     Zid+Zal  9.84 1.84   6.2318    13.46    5.36   0.000
#> 6 Zid+Did+Nev     Zid+Did  6.91 2.00   2.9767    10.84    3.45   0.001
#> 
#>  
#> Inferences of model median difference between groups ( treatment 1 - treatment 0 ) for weekc = 24 
#>  
#>   treatment 1 treatment 0  delta   SE lower.CL upper.CL t.value p.value
#> 1     Zid+Zal     Zid/Did  0.534 1.12   -1.660     2.73   0.479   0.632
#> 2     Zid+Did     Zid/Did  4.139 1.36    1.459     6.82   3.035   0.003
#> 3 Zid+Did+Nev     Zid/Did 10.135 1.66    6.874    13.40   6.108   0.000
#> 4     Zid+Did     Zid+Zal  3.605 1.44    0.768     6.44   2.498   0.013
#> 5 Zid+Did+Nev     Zid+Zal  9.601 1.73    6.204    13.00   5.554   0.000
#> 6 Zid+Did+Nev     Zid+Did  5.996 1.89    2.291     9.70   3.180   0.002
#> 
#>  
#> Inferences of model median difference between groups ( treatment 1 - treatment 0 ) for weekc = 32 
#>  
#>   treatment 1 treatment 0 delta   SE lower.CL upper.CL t.value p.value
#> 1     Zid+Zal     Zid/Did  1.35 1.04   -0.692     3.40    1.30   0.194
#> 2     Zid+Did     Zid/Did  3.00 1.20    0.633     5.37    2.49   0.013
#> 3 Zid+Did+Nev     Zid/Did  9.70 1.52    6.705    12.69    6.37   0.000
#> 4     Zid+Did     Zid+Zal  1.65 1.30   -0.907     4.20    1.27   0.206
#> 5 Zid+Did+Nev     Zid+Zal  8.34 1.60    5.206    11.48    5.23   0.000
#> 6 Zid+Did+Nev     Zid+Did  6.70 1.71    3.338    10.06    3.92   0.000

plot(res2, ylab = "CD4+1", xlab = "Week", verbose = TRUE)
```

<img src="man/figures/README-example-1.png" width="100%" />

    #> Analysis information:
    #>   Covariance structure: "UN" 
    #>   Robust inference: TRUE 
    #>   Empirical small sample adjustment: TRUE 
    #> 
    #> Error bar: 95% confidence interval

#' Applying the Box-Cox Transformation to a numeric vector.
#'
#' \code{bct.v} returns the Box-Cox transformed numeric vector (Box and Cox,
#' 1964).
#'
#' @param y a positive real number vector.
#' @param lmdint a vector containing the end-points of the interval to be
#'   searched for a transformation parameter. Default is c(-3, 3).
#'
#' @return \code{bct.v} returns the Box-Cox transformed numeric vector and
#' estimated transformation parameter.
#' \describe{
#'   \item{\code{transformed}}{The Box-Cox transformed numeric vector.}
#'   \item{\code{lambda}}{a numeric value of the estimate of the transformation
#'         parameter.}
#' }
#'
#' @references Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
#' (with discussion).
#' \emph{Journals of the Royal Statistical Society, Series B}, 26,
#' 211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
#'
#' @examples
#'   y <- exp(rnorm(50))
#'   bct.v(y)
#'
#' @export
bct.v <- function(y, lmdint = c(-3, 3)) {
  if (!is.numeric(y)) {
    stop("The argument must be numeric.")
  }
  if (sum(y <= 0) != 0) {
    stop("The argument includes non-positive value(s)")
  }
  if (sum(is.na(y)) > 0) {
    y <- y[!is.na(y)]
    warning("Missing values are excluded.")
  }
  lambda <- bcreg(y ~ 1, data.frame(y = y), lmdint)$lambda
  z <- bct(y, lambda)
  return(list(transformed = z, lambda = lambda))
}

#' Linear regression model with the Box-Cox Transformation.
#'
#' \code{bcreg} returns the maximum likelihood estimates for parameters of
#' the linear regression models with the Box-Cox transformation (Box and Cox,
#' 1964).
#'
#' @param formula a two-sided linear formula object describing the model, with
#'   the response on the left of a ~ operator and the terms, separated by +
#'   operators, on the right.
#' @param data a data frame in which to interpret the variables named in the
#'   formula.
#' @param lmdint a vector containing the end-points of the interval to be
#'   searched for a transformation parameter. Default is c(-3, 3).
#'
#' @return bcreg returns a list including following components:
#' \describe{
#'   \item{\code{lambda}}{a numeric value with the estimate of
#'         the transformation parameter.}
#'   \item{\code{beta}}{a vector with the estimates of the regression
#'         parameters.}
#'   \item{\code{sigma}}{a numeric value with the estimate of the scale
#'         parameter.}
#'   \item{\code{betainf}}{a data frame with inference results for \code{beta}
#'         under the assumption that \code{lambda} is known.}
#'   \item{\code{lik}}{a numeric value with the maximized likelihood.}
#'   \item{\code{lmObject}}{an object of "\code{lm}" containing the results of
#'         \code{lm} function on the transformed scale}
#' }
#'
#' @references Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
#' (with discussion).
#' \emph{Journals of the Royal Statistical Society, Series B}, 26,
#' 211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
#'
#' @seealso \code{\link{lm}}
#'
#' @examples
#'   data(aidscd4)
#'   #Transformation of baseline observation for aid.cd4 data
#'   bcreg(cd4.bl ~ 1, aidscd4[aidscd4$weekc == 8, ])
#'
#' @importFrom MASS ginv
#' @importFrom stats lm model.matrix optimize
#'
#' @export
bcreg <- function(formula, data, lmdint = c(-3, 3)) {
  formula <- formula(formula)
  data <- as.data.frame(data)
  y <- data[, as.character(formula)[2]]
  if (sum(y < 0, na.rm = TRUE) > 0L) {
    stop("outcome must be positive.")
  }
  options(na.action = "na.pass")
  X <- model.matrix(formula, data = data)
  if (sum(is.na(X)) > 0L) {
    stop("There are missing values in explanatory variables.")
  }
  flg.na <- !is.na(y)
  y <- y[flg.na]
  X <- X[flg.na, ]
  lbc <- function(l, y, X) {
    if (l == 0) z <- log(y)
    else z <- (y ^ l - 1) / l
    n <- length(y)
    beta <- ginv(t(X) %*% X) %*% t(X) %*% z
    zm <- z - X %*% beta
    sgm <- sqrt(sum(zm ^ 2) / n)
    lik <- -n / 2 * (log(2 * pi) + 1) - n * log(sgm) + (l - 1) * sum(log(y))
    return(lik)
  }
  res <- optimize(lbc, interval = lmdint, maximum = TRUE, y = y, X = X)
  lmd <- res$maximum
  z <- (y ^ lmd - 1) / lmd
  lik <- res$objective
  data <- as.data.frame(data[flg.na, ])
  data$z <- as.numeric(z)
  evars <- as.character(formula)[3]
  formula2 <- formula(paste("z~", evars))
  lmres <- lm(formula2, data)
  beta <- coef(lmres)
  sgm <- summary(lmres)$sigma
  names(sgm) <- "Residual"
  bcres <- as.data.frame(summary(lmres)$coefficients)
  list(lambda = lmd, beta = beta, sigma = sgm,
            betainf = bcres, logLik = lik, lmObject = lmres)
}

#' Summarize a bcmmrm Object.
#'
#' Additional information about the Box-Cox transformed MMRM analysis
#' represented by \code{object} is extracted and included as components
#' of \code{object}.
#'
#' @param object an object inheriting from class "\code{bcmmrm}", representing
#' the Box-Cox transformed MMRM analysis.
#' @param robust an optional logical value used to specify whether to apply
#' the robust inference. The default is \code{TRUE}.
#' @param ssadjust an optional logical value used to specify whether to apply
#' the empirical small sample adjustment. The default is \code{TRUE}.
#' @param ... some methods for this generic require additional arguments.
#' None are used in this method.
#'
#' @return an object inheriting from class \code{summary.bcmmrm} with all
#' components included in \code{object} (see \code{\link[nlme]{glsObject}} for
#' a full description of the components) plus the following components:
#' \describe{
#'   \item{\code{median}}{a list including inference results of the model median
#'         for specified values of \code{robust} and \code{ssadjust}.}
#'   \item{\code{meddif}}{a list including inference results of the model median
#'         difference for specified values of \code{robust} and
#'         \code{ssadjust}.}
#'   \item{\code{robust}}{a specified value of \code{robust}.}
#'   \item{\code{ssadjust}}{a specified value of \code{ssadjust}.}
#' }
#'
#' @seealso \code{\link{bcmmrm}}, \code{\link{bcmmrmObject}},
#'          \code{\link{summary}}
#'
#' @examples
#'  data(aidscd4)
#'  resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
#'                  time = weekc, id = id, structure = "AR(1)",
#'                  glabel = c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
#'  summary(resar)
#'
#' @export
summary.bcmmrm <- function(object, robust = TRUE, ssadjust = TRUE, ...) {
  if (robust & ssadjust) {
    med <- object$median.rob.adj
    dif <- object$meddif.rob.adj
  }
  if (!robust & ssadjust) {
    med <- object$median.mod.adj
    dif <- object$meddif.mod.adj
  }
  if (robust & !ssadjust) {
    med <- object$median.rob
    dif <- object$meddif.rob
  }
  if (!robust & !ssadjust) {
    med <- object$median.mod
    dif <- object$meddif.mod
  }
  structure(class = "summary.bcmmrm",
            c(object, list(median = med,
                 meddif = dif,
                 robust = robust,
                 ssadjust = ssadjust
                 )))
}

#' @export
print.summary.bcmmrm <- function(x, digits = 3, ...) {
    mCall <- x$call
    covstr <- mCall$structure
    if (is.null(covstr) & !is.null(mCall$id)) {
      covstr <- "UN"
    }
    cat("Model median inference based on MMRM with Box-Cox transformation\n")
    cat("\nData and variable information:\n")
    cat("  Outcome:", deparse(mCall$outcome), "\n")
    cat("  Group:", deparse(mCall$group), "\n")
    cat("  Time:", deparse(mCall$time), "\n")
    cat("  ID:", deparse(mCall$id), "\n")
    cat("  Covariate(s):", deparse(mCall$covv), "\n")
    cat("  Data:", deparse(mCall$data), "\n")
    cat("\nAnalysis information:\n")
    cat("  Covariance structure:", deparse(covstr), "\n")
    cat("  Robust inference:", x$robust, "\n")
    cat("  Empirical small sample adjustment:", x$ssadjust, "\n")
    cat("  Confidence level:", x$conf.level, "\n")
    cat("\nAnalysis results:\n")
    cat("  Estimated transformation parameter: ",
        format(x$lambda, digits = digits), "\n")
    nt <- length(x$median)
    ng <- nrow(x$median[[1]])
    group0 <- x$meddif[[1]]$group0
    glabel0 <- as.character(group0)
    group1 <- x$meddif[[1]]$group1
    glabel1 <- as.character(group1)
    for (g in 1:ng) {
      if (sum(group0 == g) != 0) {
        glabel0[group0 == g] <- as.character(x$group.tbl$label[g])
      }
      if (sum(group1 == g) != 0) {
        glabel1[group1 == g] <- as.character(x$group.tbl$label[g])
      }
    }

    for (t in 1:nt) {
      medt <- x$median[[t]]
      names(medt)[1] <- deparse(mCall$group)
      medt[, 1] <- x$group.tbl$label
      cat("\n", "\nModel median inferences for", deparse(x$call$time), "=",
          x$time.tbl$label[t], "\n", "\n")
      print(medt, digits = digits, ...)
    }
    for (t in 1:nt) {
      dift <- x$meddif[[t]]
      names(dift)[1] <- paste(deparse(mCall$group), "1")
      names(dift)[2] <- paste(deparse(mCall$group), "0")
      dift[, 1] <- glabel1
      dift[, 2] <- glabel0
      cat("\n", "\nInferences of model median difference between groups (",
          names(dift)[1], "-", names(dift)[2], ") for", deparse(x$call$time),
          "=", x$time.tbl$label[t], "\n", "\n")
      dift[, 8] <- round(dift[, 8], digits = digits)
      print(dift, digits = digits, ...)
    }
    invisible(x)
  }

#' Applying the Box-Cox Transformation.
#'
#' \code{bct} returns the Box-Cox transformed numeric vector (Box and Cox,
#' 1964).
#'
#' @param y a positive real number vector.
#' @param lambda a scalar transformation parameter.
#'
#' @return bct returns the Box-Cox transformed numeric vector,
#' \code{z = log(y)} for \code{lambda = 0},
#' \code{z = (y ^ lambda - 1) / lambda} for \code{lambda ne 1}.
#'
#' @references Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
#' (with discussion).
#' \emph{Journals of the Royal Statistical Society, Series B}, 26,
#' 211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
#'
#' @examples
#'   y <- exp(rnorm(10))
#'   z <- bct(y, 0) #log transformation
#'
#' @export
bct <- function(y, lambda) {
  if (sum(y < 0)) {
    stop("All values in y must be positive.")
  }
  if (lambda == 0) {
    z <- log(y)
  }
  else {
    z <- (y ^ lambda - 1) / lambda
  }
  return(z)
}

#' Plot a bcmmrm Object.
#'
#' Plot for the model medians of each treatment groups with the 95 percent
#' confidence intervals stored in \code{bcmmrmObject}.
#'
#' @param x an object inheriting from class "\code{bcmmrm}", representing
#' the Box-Cox transformed MMRM analysis.
#' @param robust an optional logical value used to specify whether to apply
#' the robust inference. The default is \code{TRUE}.
#' @param ssadjust an optional logical value used to specify whether to apply
#' the empirical small sample adjustment. The default is \code{TRUE}.
#' @param tnom a optional logical value indicating the scale of x axis
#' of the longitudinal median plot. When \code{tnom} is \code{TRUE}, nominal
#' scale is used and widths between any combinations of neighbor time points
#' are same. When \code{tnom} is \code{FALSE}, actual scale of \code{time}
#' variable is used. The default is \code{TRUE}.
#' @param dt an numeric value indicating shift length between groups
#'  in the longitudinal median plot. A multiplying factor for the default
#'  settings specified (e.g. if \code{2} is specified,
#'  shift length is twice longer than that for the default setting).
#'  The default is \code{1}.
#' @param timepoint an numeric value of a specified level of \code{time}
#' variable at which median plot is created. When \code{timepoint} is
#' \code{NULL} and number of time points is not 1, longitudinal median plot
#' (x axis is \code{time}) is created. Otherwise, median plot where
#' x axis is \code{group} is created. The default is \code{NULL}.
#' @param xlab a title for the x axis. The default is \code{NULL} and
#'  the name of \code{time} or \code{group} variable is used.
#' @param ylab a title for the y axis. The default is \code{NULL} and
#'  the name of \code{outcome} variable is used.
#' @param xlim a numeric vector with length of 2 indicating limits of x
#'  axis. The default is \code{NULL} and limits are calculated automatically.
#' @param ylim a numeric vector with length of 2 indicating limits of y
#'  axis. The default is \code{NULL} and limits are calculated automatically.
#' @param lwd an optional positive numeric value indicating line width.
#' The default is \code{2}.
#' @param col an integer or a character vector with length of the number of
#'  groups indicating colors of lines for each treatment group. The default is
#'  \code{NULL} and all of colors are black.
#' @param lty an optional integer or a character vector with length of the
#'  number of groups indicating line types of lines for each treatment group.
#'  The default is \code{NULL} and \code{1:ng} is used, where ng is number of
#'  groups.
#' @param main a main title for the plot. The default is \code{TRUE} and
#'  default title is "(Longitudinal) Plot for median of each group".
#' @param sub a sub title for the plot. The default is \code{NULL}.
#' @param legend a logical optional value specifying to add legends to
#'  plots. When \code{legend} is \code{TRUE} legends are added to the plot.
#'  Otherwise, legends are not added.  The default is \code{TRUE}.
#' @param loc a character value indicating the location of the legends.
#' The location can be specified by setting \code{loc} to a single keyword
#' from the list \code{"bottomright"}, \code{"bottom"}, \code{"bottomleft"},
#' \code{"left"}, \code{"topleft"}, \code{"top"}, \code{"topright"},
#' \code{"right"}, and \code{"center"}. The default is \code{"topright"}.
#' @param  verbose a logical optional value specifying to print the detailed
#' analysis information in the console. The default is \code{FALSE}.
#' @param ... some methods for this generic require additional arguments.
#'
#' @return a median plot.
#'
#' @seealso \code{\link{bcmmrm}}, \code{\link{bcmmrmObject}}
#'
#' @examples
#'  data(aidscd4)
#'  resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
#'                 time = weekc, id = id, structure = "AR(1)", glabel =
#'                 c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
#'  plot(resar, xlab = "Week", ylab = "CD4+1", col = 1:4, main = NULL)
#'  plot(resar, timepoint = 32, xlab = "Treatment", ylab = "CD4+1")
#' @importFrom graphics plot grid lines arrows axis legend
#' @export
plot.bcmmrm <- function(x, robust = TRUE, ssadjust = TRUE, dt = 1,
                        timepoint = NULL, tnom = TRUE, xlab = NULL,
                        ylab = NULL, xlim = NULL, ylim = NULL,
                        lwd = 2, col = NULL, lty = NULL,
                        main = TRUE, sub = NULL, legend = TRUE,
                        loc = "topright", verbose = FALSE, ...) {
  x <- summary(x, robust = robust, ssadjust = ssadjust)
  med <- x$median
  if (is.null(ylab)) {
    ylab <- deparse(x$call$outcome)
  }
  ng <- nrow(med[[1]])
  if (is.null(col)) {
    col <- rep("black", ng)
  }
  glabel <- x$group.tbl$label
  if (length(med) == 1 | !is.null(timepoint)) {
    if (!is.null(timepoint)) {
      med <- med[[x$time.tbl$code[x$time.tbl$label == timepoint]]]
    } else {
      med <- med[[1]]
    }

    Group <- 1:ng
    if (is.null(ylim)) {
      upl <- max(med$upper.CL) * 1.2
      lol <- min(med$lower.CL) * 0.8
      ylim <- c(lol, upl)
    }
    if (is.null(xlim)) {
      xlim <- c(0.5, ng + 0.5)
    }

    if (is.null(xlab)) {
      xlab <- deparse(x$call$group)
    }
    if (!is.null(main)) {
      if (main == TRUE) {
        main <- "Plot for model median of each group"
      }
    }

    plot(1, 1, xlim = xlim, ylim = ylim, type = "n", ylab = ylab, xlab = xlab,
         xaxt = "n",
         panel.first = grid(NA, NULL, lty = 2, col = "#E9DECA", ...),
         main = main, sub = sub)
    axis(1, at = Group, labels = glabel)

    points(Group, med$median, lwd = lwd, col = col)
    arrows(Group, med$upper.CL, Group, med$lower.CL,
           code = 3, angle = 90, length = .07, lwd = lwd, col = col)
    if (verbose) {
      cat("Analysis information:\n")
      cat("  Covariance structure:", deparse(x$call$structure), "\n")
      cat("  Robust inference:", x$robust, "\n")
      cat("  Empirical small sample adjustment:", x$ssadjust, "\n")
      cat("\nError bar:", x$conf.level * 100, "% confidence interval\n")
      cat("\nTimepoint:", deparse(x$call$time), "=", timepoint)
    }
  } else {
    Time2 <- x$time.tbl$label
    Time1 <- x$time.tbl$code
    nt <- max(Time1)
    if (tnom == 1) {
      Time0 <- Time1
      hh <- 1
    } else {
      Time0 <- Time2
      hh <- (Time2[nt] - Time2[1]) / nt
    }
    Time <- list()
    est <- list()
    upper <- list()
    lower <- list()
    for (i in 1:ng) {
      Time[[i]] <- Time0 + 0.05 * dt * hh * (2 * i - ng - 1)
      est0 <- c()
      upper0 <- c()
      lower0 <- c()
      for (j in 1:nt) {
        est0 <- c(est0, med[[j]]$median[i])
        upper0 <- c(upper0, med[[j]]$upper.CL[i])
        lower0 <- c(lower0, med[[j]]$lower.CL[i])
      }
      est[[i]] <- est0
      upper[[i]] <- upper0
      lower[[i]] <- lower0
    }
    up <- c()
    lo <- c()
    for (i in 1:nt) {
      up <- c(up, med[[i]]$upper.CL)
      lo <- c(lo, med[[i]]$lower.CL)
    }
    if (is.null(ylim)) {
      upl <- max(up) * 1.2
      lol <- min(lo) * 0.8
      ylim <- c(lol, upl)
    }
    if (is.null(xlim)) {
      if (tnom == 1) {
        xlim <- c(0.5, nt + 0.5)
      } else {
        lx <- min(Time2)
        ux <- max(Time2)
        dux <- lx  * 0.1
        xlim <- c(min(c(lx - dux, lx - 1)), max(c(ux + dux, ux + 1)))
      }
    }
    if (is.null(xlab)) {
      xlab <- deparse(x$call$time)
    }
    if (!is.null(main)) {
      if (main == T) {
        main <- "Longitudinal plot for model median of each group"
      }
    }
    plot(1, 1, xlim = xlim, ylim = ylim, type = "n", ylab = ylab, xlab = xlab,
         xaxt = "n",  panel.first = grid(NA, NULL, lty = 2, col = "#E9DECA"),
         main = main, sub = sub, ...)
    if (tnom == 1) {
      axis(1, at = Time0, labels = Time2)
    } else {
      axis(1, at = Time0)
    }
    if (is.null(lty)) {
      lty <- 1:ng
    }
    for (i in 1:ng) {
      lines(Time[[i]], est[[i]], lty = lty[i], lwd = lwd, col = col[i])
      arrows(Time[[i]], upper[[i]], Time[[i]], lower[[i]],
             code = 3, angle = 90, length = .07 * dt, lwd = lwd,
             col = col[i])
    }
    if (legend) {
      legend(loc, legend = glabel, lty = lty, col = col, lwd = lwd)
    }
    if (verbose) {
      cat("Analysis information:\n")
      cat("  Covariance structure:", deparse(x$call$structure), "\n")
      cat("  Robust inference:", x$robust, "\n")
      cat("  Empirical small sample adjustment:", x$ssadjust, "\n")
      cat("\nError bar: 95% confidence interval\n")
    }
  }
}

#' Fitted bcmmrm Object
#'
#' An object returned by the \code{\link{bcmmrm}} function, inheriting from
#' class "bcmmrm" and representing the Box-Cox transformed MMRM analysis.
#' Objects of this class have methods for the generic functions \code{boxplot},
#' \code{coef}, \code{logLik}, \code{plot}, \code{print}, and \code{summary}.
#'
#' @name bcmmrmObject
#'
#' @return The following components must be included in a legitimate "bcmmrm"
#' object.\describe{
#'     \item{\code{call}}{a list containing an image of the \code{bcmmrm}
#'           call that produced the object.}
#'     \item{\code{median.mod}, \code{median.rob}, \code{median.mod.adj},
#'           \code{median.rob.adj}}{lists including inference results for the
#'           model medians for all groups. Levels of the list are time points,
#'           where correspondence table is given as \code{time.tbl$code}.
#'           mod: model-based inference, rob: robust inference,
#'           adj: with empirical small sample adjustment.}
#'     \item{\code{meddif.mod}, \code{meddif.rob}, \code{meddif.mod.adj},
#'           \code{meddif.rob.adj}}{lists including inference results for the
#'           for the model median differences between all pairs of groups
#'           (group1 - group0). Levels of the list are time points,
#'           where correspondence table is given as \code{time.tbl$code}.
#'           mod: model-based inference, rob: robust inference,
#'           adj: with empirical small sample adjustment.}
#'     \item{\code{lambda}}{a numeric value of estimates of the transformation
#'           parameter.}
#'     \item{\code{R}}{a correlation matrix for transformed outcomes.}
#'     \item{\code{betainf}}{inference results for beta under the assumption
#'           that lambda is known. Note that standard errors might be
#'           underestimated although statistical tests would be asymptotically
#'           valid.}
#'     \item{\code{time.tbl}}{a data frame of a correspondence table for
#'           the timepoints.}
#'     \item{\code{group.tbl}}{a data frame of a correspondence table for
#'           treatment groups.}
#'     \item{\code{inf.marg}}{a list with results of \code{\link{bcmarg}}
#'           function.}
#'     \item{\code{outdata}}{a data frame where the transformed outcome
#'           (\code{ytr}), the fitted values on the transformed scale
#'           (\code{ytr.fitted}), and the residuals on the transformed scale
#'           (\code{res.tr}) are added to input data.}
#'     \item{\code{conf.level}}{a numeric value of the specified confidence
#'           level.}
#'   }
#'
#' @seealso \code{\link{bcmmrm}}
NULL

#' Marginal Model of the Mixed Effect Model with the Box-Cox Transformation.
#'
#' \code{bcmarg} returns the inference results the parameters of
#'   the marginal model of the linear mixed effect model with the Box-Cox
#'   transformation proposed by Maruo et al. (2017). If time and id are not
#'   specified, inference results reduce to the results for the context of
#'   linear regression model provided by Maruo et al. (2015).
#'
#' @param formula a two-sided linear formula object describing the model, with
#'   the response on the left of a ~ operator and the terms, separated by +
#'   operators, on the right.
#' @param data a data frame containing the variables used in the model.
#' @param time time variable name for repeated measurements. The default is
#'   NULL.
#' @param id subject id variable name for repeated measurements. The default is
#'   NULL.
#' @param structure specify the covariance structure from c("UN", "CS",
#'   "AR(1)"). The default is "UN".
#' @param lmdint a vector containing the end-points of the interval to be
#'   searched for a transformation parameter. The default is c(-3, 3).
#'
#' @return an object of class "\code{bcmarg}". Objects of this class have
#' methods for the generic functions  \code{coef}, \code{logLik},
#' \code{print}, and \code{summary}. The object includes following components
#' for the marginal model parameter inference:
#' \describe{
#'   \item{\code{lambda}}{a numeric value of the estimate of the transformation
#'         parameter.}
#'   \item{\code{beta}}{a vector with the estimates of the regression
#'         parameters.}
#'   \item{\code{alpha}}{a vector with the estimates of the covariance
#'         parameters.}
#'   \item{\code{V}}{variance-covariance matrix for any subject with no missing
#'         values.}
#'   \item{\code{betainf}}{a matrix containing the inference results for
#'        \code{beta} under the assumption that lambda is known.
#'        Note that standard errors might be underestimated
#'        although statistical tests would be asymptotically valid.}
#'   \item{\code{Vtheta.mod}}{model-based variance-covariance matrix for MLE of
#'         the vector of all parameters: \code{c(lambda, beta, alpha)}.}
#'   \item{\code{Vtheta.rob}}{robust variance-covariance matrix for MLE of
#'         the vector of all parameters.}
#'   \item{\code{logLik}}{a numeric value of the maximized likelihood.}
#'   \item{\code{adj.prm}}{a vector with parameters used for the empirical
#'         small sample adjustment in \code{\link{bcmmrm}}:
#'         c(number of subjects, number of completed subjects, number of
#'         outcome observations, number of missing observations).}
#'   \item{\code{glsObject}}{an object of "\code{gls}" (or "\code{lm}" when
#'         \code{time} and \code{id} are not specified) containing results of
#'         \code{gls} (or \code{lm}) function on the transformed scale. }
#' }
#'
#' @references \itemize{
#'   \item Maruo, K., Isogawa, N., Gosho, M. (2015). Inference of median
#'         difference based on the Box-Cox model in randomized clinical trials.
#'         \emph{Statistics in Medicine}, 34, 1634-1644,
#'         \doi{10.1002/sim.6408}.\cr
#'   \item Maruo, K., Yamaguchi, Y., Noma, H., Gosho, M. (2017). Interpretable
#'         inference on the mixed effect model with the Box-Cox transformation.
#'         \emph{Statistics in Medicine}, 36, 2420-2434,
#'         \doi{10.1002/sim.7279}.
#' }
#'
#' @seealso \code{\link{bcmmrm}} \code{\link[nlme]{gls}}
#'
#' @examples
#'  data(aidscd4)
#'  bcmarg(cd4 ~ as.factor(treatment) * as.factor(weekc) + age,
#'         data = aidscd4, time = weekc, id = id, structure = "AR(1)")
#'
#' @importFrom nlme gls glsControl corSymm varIdent corCompSymm corAR1 corMatrix
#' @importFrom MASS ginv
#' @importFrom stats coef ftable model.matrix na.omit optimize xtabs predict
#'
#' @export
bcmarg <- function(formula, data, time = NULL, id = NULL, structure = "UN",
                   lmdint = c(-3, 3)) {
  backup_options <- options()
  Call <- match.call()
  tr <- function(X) sum(diag(X))
  covcalcUN <- function(glsob, nt) {
    flg <- 0
    j <- 0
    while (flg == 0) {
      j <- j + 1
      corm <- corMatrix(glsob$modelStruct$corStruct)[[j]]
      if (nrow(corm) == nt) flg <- 1
    }
    varstruct <- glsob$modelStruct$varStruct
    varests <- coef(varstruct, uncons = FALSE, allCoef = TRUE)
    covm <- corm * glsob$sigma ^ 2 * t(t(varests)) %*% t(varests)
    return(covm)
  }
  covcalcCSAR <- function(glsob, nt) {
    flg <- 0
    j <- 0
    while (flg == 0) {
      j <- j + 1
      corm <- corMatrix(glsob$modelStruct$corStruct)[[j]]
      if (nrow(corm) == nt) flg <- 1
    }
    covm <- corm * glsob$sigma ^ 2
    return(covm)
  }
  dec2bin <- function(num, digit=0) {
    if (num <= 0 && digit <= 0) {
      return(NULL)
    }else{
      return(append(Recall(num %/% 2, digit - 1), num %% 2))
    }
  }
  likn <- function(formula, data, structure, lmd) {
    y <- data$y
    if (lmd == 0) data$z <- log(y)
    else data$z <- (y ^ lmd - 1) / lmd
    if (structure == "UN") {
      RS <- gls(model = formula, data = data,
                correlation = corSymm(form = ~ time | id),
                weights = varIdent(form = ~ 1 | time), method = "ML",
                control = glsControl(msMaxIter = 100), na.action = na.omit)
    }
    if (structure == "CS") {
      RS <- gls(model = formula, data = data,
                correlation = corCompSymm(form = ~ time | id),
                method = "ML", control = glsControl(msMaxIter = 100),
                na.action = na.omit)
    }
    if (structure == "AR(1)") {
      RS <- gls(model = formula, data = data,
                correlation = corAR1(form = ~ time | id),
                method = "ML", control = glsControl(msMaxIter = 100),
                na.action = na.omit)
    }
    return(RS$logLik[1] + (lmd - 1) * sum(log(y), na.rm = TRUE))
  }

  formula <- formula(formula)
  yc <- as.character(formula)[2]

  options(na.action = "na.pass")
  X <- model.matrix(formula, data = data)

  if (sum(is.na(X)) > 0L) {
    stop("There are missing values in explanatory variables.")
  }
  if ((deparse(substitute(time)) == "NULL" &
       deparse(substitute(id)) != "NULL") |
      (deparse(substitute(time)) != "NULL" &
       deparse(substitute(id)) == "NULL")) {
    stop("If either time or id is specified,
         both of time and id must be specified.")
  }
  data$y <- data[, yc]

  if (sum(data$y < 0, na.rm = TRUE) > 0L) {
    stop("outcome must be positive.")
  }
  if (deparse(substitute(time)) == "NULL" &
      deparse(substitute(id)) == "NULL") {
    try1 <- bcreg(formula, data)
    nt <- 1
    data$id <- as.character(seq_len(nrow(data)))
    msflg <- table(data$id, is.na(data$y))[, 1]
    N <- sum(msflg != 0)
  } else {
    evars <- as.character(formula)[3]
    timec <- deparse(substitute(time))
    data$id <- as.character(data[, deparse(substitute(id))])
    ttbl <- as.numeric(names(table(data[, timec])))
    nt <- length(ttbl)
    if (!structure %in% c("UN", "CS", "AR(1)")) {
      stop("Select structure from c(\"UN\", \"CS\", \"AR(1)\")")
    }
    time2 <- data[, timec]
    for (i in 1:nt) {
      time2[data[, timec] == ttbl[i]] <- i
    }
    data$time <- time2
    msflg <- table(data$id, is.na(data$y))[, 1]
    N <- sum(msflg != 0)
    omis <- names(msflg)[msflg == 0]
    if (length(omis) > 0L) {
      for (i in seq_len(length(omis))) {
        data <- data[data$id != omis[i], ]
      }
    }
    data0 <- data[!duplicated(data$id), ]
    data0$y <- NULL
    data01 <- data0
    data1 <- c()
    for (i in 1:nt) {
      data01$time <- i
      data1 <- rbind(data1, data01)
    }
    data2 <- data[, c("id", "time", "y")]
    data <- merge(data2, data1, by = c("id", "time"), all = TRUE)
    for (i in 1:nt) {
      data[data$time == i, timec] <- ttbl[i]
    }

    if (nt == 1) {
      try1 <- bcreg(formula, data, lmdint)
    } else {
      formula2 <- formula(paste("z~", evars))
      try1 <- optimize(likn, interval = lmdint, maximum = TRUE,
                       formula = formula(paste("z~", evars)), data = data,
                       structure = structure)
    }
  }

  if (nt == 1) {
    le <- try1$lambda
    beta <- try1$beta
    alp <- try1$sigma ^ 2
    names(alp) <- "Residual"
    V <- as.matrix(alp)
    data$z <- (data$y ^ le - 1) / le
    bcres <- try1$betainf
    structure <- "UN"
    lik <- try1$logLik
    RS <- try1$lmObject
  } else {
    le <- try1$maximum
    lik <- try1$objective

    data$z <- (data$y ^ le - 1) / le

    if (structure == "UN") {
      RS <- gls(model = formula2, data = data,
                correlation = corSymm(form = ~ time | id),
                weights = varIdent(form = ~ 1 | time), method = "ML",
                control = glsControl(msMaxIter = 100), na.action = na.omit)
      RS$call$model <- eval(formula2)
      V <- covcalcUN(RS, nt)
      alp <- c()
      nmalp <- c()
      for (i in 1:nt) {
        for (j in i:nt) {
          alp <- c(alp, V[i, j])
          nmalp <- c(nmalp, paste("UN(", i, ",", j, ")", sep = ""))
        }
        names(alp) <- nmalp
      }
    }
    if (structure == "CS") {
      RS <- gls(model = formula2, data = data,
                correlation = corCompSymm(form = ~ time | id),
                method = "ML", control = glsControl(msMaxIter = 100),
                na.action = na.omit)
      RS$call$model <- eval(formula2)
      V <- covcalcCSAR(RS, nt)
      alp <- c(V[1, 2], V[1, 1] - V[1, 2])
      names(alp) <- c("ID", "Residual")
    }
    if (structure == "AR(1)") {
      RS <- gls(model = formula2, data = data,
                correlation = corAR1(form = ~ time | id),
                method = "ML", control = glsControl(msMaxIter = 100),
                na.action = na.omit)
      RS$call$model <- eval(formula2)
      V <- covcalcCSAR(RS, nt)
      alp <- c(V[1, 1], V[1, 2] / V[1, 1])
      names(alp) <- c("Variance", "Correlation")
    }
    beta <- RS$coefficients
    bcres <- summary(RS)$tTable
    time.tbl <- sort(unique(data[, deparse(substitute(time))]))
    rownames(V) <- time.tbl
    colnames(V) <- time.tbl
  }
  ns <- length(alp)
  msflg <- table(data$id, is.na(data$y))[, 1]
  N <- sum(msflg != 0)
  omis <- names(msflg)[msflg == 0]
  idt <- cbind((1:N) %x% (numeric(nt) + 1), (numeric(N) + 1) %x% (1:nt))

  if (length(omis) > 0L) {
    for (i in seq_len(length(omis))) {
      data <- data[data$id != omis[i], ]
    }
  }
  Ncc <- sum(msflg == nt)
  flg.na <- !is.na(data$y)
  n.data <- sum(flg.na)
  X <- model.matrix(formula, data = data)
  y <- data$y[flg.na]
  z <- data$z[flg.na]
  n.na <- length(flg.na) - n.data
  adj.prm <- c(N, Ncc, n.data, n.na)
  nb <- length(beta)
  Xb <- list()
  idt <- cbind((1:N) %x% (numeric(nt) + 1), (numeric(N) + 1) %x% (1:nt))
  for (j in 1:nb) {
    dum <- as.data.frame(cbind(idt, X[, j]))
    names(dum) <- c("a", "b", "c")
    Xb[[j]] <- as.matrix(ftable(xtabs(c ~ ., dum)))
  }
  dimnames(X) <- NULL
  nti <- table(data$id[!is.na(data$y)])
  ntic <- cumsum(nti)
  ntic <- c(0, ntic)
  Xflg <- (1:N) %x% (numeric(nt) + 1)
  X <- X[flg.na, ]
  Xflg <- Xflg[flg.na]
  ly <- log(y)
  dz <- le ^ (-2) * (y ^ le * (le * ly - 1) + 1)
  ddz <- le ^ (-1) * y ^ le * ly ^ 2 - 2 * le ^ (-2) * y ^ le * ly +
    2 * le ^ (-3) * (y ^ le - 1)
  if (!is.matrix(X)) {
    X <- as.matrix(X)
  }
  re <- z - X %*% beta
  dum <- as.data.frame(cbind(idt[flg.na, ], re))
  names(dum) <- c("a", "b", "c")
  rt <- as.matrix(xtabs(c ~ ., dum))
  dum <- as.data.frame(cbind(idt[flg.na, ], dz))
  names(dum) <- c("a", "b", "c")
  dzt <- as.matrix(xtabs(c ~ ., dum))
  dum <- as.data.frame(cbind(idt[flg.na, ], ddz))
  names(dum) <- c("a", "b", "c")
  ddzt <- as.matrix(xtabs(c ~ ., dum))
  dum <- as.data.frame(cbind(idt[flg.na, ], ly))
  names(dum) <- c("a", "b", "c")
  lyt <- as.matrix(xtabs(c ~ ., dum))
  lysum <- rowSums(lyt)
  dp <- numeric(N)
  t2 <- 2 ^ nt - 1
  nsf <- c()
  for (j1 in 1:nt) {
    for (j2 in j1:nt) {
      nsf <- rbind(nsf, t(c(j1, j2)))
    }
  }
  colf <- list()
  for (j in 1:(t2)) {
    mv <- dec2bin(j - 1, nt)
    aprt <- as.matrix(apply(rt == 0, 1, "==", mv))
    if (t2 == 1) {
      aprt <- t(aprt)
    }
    dp[colSums(aprt) == nt] <- j
    colf[[j]] <- (1:nt)[mv == 0]
  }
  ndp <- numeric(t2)
  dpf <- list()
  for (j in 1:t2) {
    ndp[j] <- sum(dp == j)
    dpf[[j]] <- (dp == j)
  }
  Hl <- 0
  Jl <- 0
  Hb <- matrix(0, nb, nb)
  Hs <- matrix(0, ns, ns)
  Hlb <- matrix(0, 1, nb)
  Hls <- matrix(0, 1, ns)
  Hbs <- matrix(0, nb, ns)
  Jb <- matrix(0, nb, nb)
  Js <- matrix(0, ns, ns)
  Jlb <- matrix(0, 1, nb)
  Jls <- matrix(0, 1, ns)
  Jbs <- matrix(0, nb, ns)
  for  (j in 1:t2) {
    if (ndp[j] != 0) {
      rj <- rt[dp == j, colf[[j]], drop = FALSE]
      nj <- nrow(rj)
      lysj <- lysum[dp == j]
      S <- V[colf[[j]], colf[[j]], drop = FALSE]
      nSj <- nrow(S)
      cholS <- chol(S)
      mat0 <- matrix(0, nSj, nSj)
      dS <- list()
      dS2 <- list()
      if (structure == "UN") {
        for (j1 in 1:ns) {
          dS[[j1]] <- 1 * (S == alp[j1])
          dS2[[j1]] <- list()
          for (j2 in 1 : ns) {
            dS2[[j1]][[j2]] <- mat0
          }
        }
      }
      if (structure == "CS") {
        dS[[1]] <- diag(nSj)
        dS[[2]] <- matrix(1, nSj, nSj)
        for (j1 in 1:2) {
          dS2[[j1]] <- list()
          for (j2 in 1:2) {
            dS2[[j1]][[j2]] <- mat0
          }
        }
      }
      if (structure == "AR(1)") {
        dS[[1]] <- S / S[1, 1]
        dS[[2]] <- mat0
        for (j1 in 1:2) {
          dS2[[j1]] <- list()
          for (j2 in 1:2) {
            dS2[[j1]][[j2]] <- mat0
          }
        }
        for (j1 in 1:nSj) {
          for (j2 in 1:nSj) {
            rp <- abs(j1 - j2)
            dS[[2]][j1, j2] <- rp * alp[1] * alp[2] ^ (rp - 1)
            dS2[[1]][[2]][j1, j2] <- rp * alp[2] ^ (rp - 1)
            dS2[[2]][[1]][j1, j2] <- rp * alp[2] ^ (rp - 1)
            dS2[[2]][[2]][j1, j2] <- rp * (rp - 1) * alp[1] *
              alp[2] ^ (rp - 2)
          }
        }
      }
      iS <- solve(S)
      sSr <- backsolve(cholS, t(rj), transpose = TRUE)
      sxb <- list()
      sdz <- list()
      sddz <- list()
      xjb <- list()
      cs_xSr <- list()
      dzj <- dzt[dp == j, colf[[j]], drop = FALSE]
      ddzj <- ddzt[dp == j, colf[[j]], drop = FALSE]
      sdz <- backsolve(cholS, t(dzj), transpose = TRUE)
      sddz <- backsolve(cholS, t(ddzj), transpose = TRUE)
      cs_zSr <- colSums(sdz * sSr)
      for (jb in 1:nb) {
        xj <- Xb[[jb]][dp == j, colf[[j]], drop = FALSE]
        xjb[[jb]] <- xj
        sxb0 <- backsolve(cholS, t(xj), transpose = TRUE)
        sxb[[jb]] <- sxb0
        cs_xSr[[jb]] <- colSums(sxb0 * sSr)
      }
      qAr <- list()
      cs_rAr <- list()
      rAA <- list()
      AA <- list()
      tris <- numeric(ns)
      for (js in 1:ns) {
        j1 <- nsf[js, 1]
        j2 <- nsf[js, 2]
        if (sum(colf[[j]] == j1) == 1 & sum(colf[[j]] == j2) == 1) {
          AA0 <- -iS %*% dS[[js]] %*% iS
          qr0 <- qr(AA0)
          AA[[js]] <- AA0
          rAA[[js]] <- qr.R(qr0)
          qx0 <- t(qr.Q(qr0)) %*% t(rj)
          rx0 <- qr.R(qr0) %*% t(rj)
          tris[js] <- tr(iS %*% dS[[js]])
          qAr[[js]] <- qx0
          cs_rAr[[js]] <- colSums(qx0 * rx0)
        } else {
          qAr[[js]] <- NA
        }
      }
      Hl <- Hl - sum(sddz * sSr) - sum(sdz ^ 2)
      Jl <- Jl + sum(lysj ^ 2) + sum(cs_zSr ^ 2) - 2 * sum(lysj * cs_zSr)
      for (j1 in 1:nb) {
        for (j2 in j1:nb) {
          if (!is.na(sxb[[j1]][1]) & !is.na(sxb[[j2]][1])) {
            Hb[j1, j2] <- Hb[j1, j2] - sum(sxb[[j1]] * sxb[[j2]])
            Jb[j1, j2] <- Jb[j1, j2] + sum(cs_xSr[[j1]] * cs_xSr[[j2]])
          }
        }
      }
      for (j1 in 1:ns) {
        for (j2 in j1:ns) {
          if (!is.na(qAr[[j1]][1]) & !is.na(qAr[[j2]][1])) {
            AA2 <- iS %*% (2 * dS[[j1]] %*% iS %*% dS[[j2]] -
                             dS2[[j1]][[j2]]) %*% iS
            qr0 <- qr(AA2)
            qx0 <- t(qr.Q(qr0)) %*% t(rj)
            rx0 <- qr.R(qr0) %*% t(rj)
            Hs[j1, j2] <- Hs[j1, j2] - 0.5 * nj *
              tr(AA[[j1]] %*% dS[[j2]] + iS %*% dS2[[j1]][[j2]]) -
              0.5 * sum(qx0 * rx0)
            Js[j1, j2] <- Js[j1, j2] + 0.25 * nj * tris[j1] * tris[j2] +
              0.25 * sum(cs_rAr[[j1]] * cs_rAr[[j2]]) +
              0.25 * sum(tris[j1] * cs_rAr[[j2]]) +
              0.25 * sum(tris[j2] * cs_rAr[[j1]])
          }
        }
      }
      for (j2 in 1:nb) {
        Hlb[1, j2] <- Hlb[1, j2] + sum(sdz * sxb[[j2]])
        Jlb[1, j2] <- Jlb[1, j2] + sum(lysj * cs_xSr[[j2]]) -
          sum(cs_zSr * cs_xSr[[j2]])
      }
      for (j2 in 1:ns) {
        if (!is.na(qAr[[j2]][1])) {
          dzrA <- rAA[[j2]] %*% t(dzj)
          Hls[1, j2] <- Hls[1, j2] - sum(dzrA * qAr[[j2]])
          Jls[1, j2] <- Jls[1, j2] - 0.5 * sum(lysj * tris[j2]) -
            0.5 * sum(lysj * cs_rAr[[j2]]) +
            0.5 * sum(cs_zSr * tris[j2]) + 0.5 * sum(cs_zSr * cs_rAr[[j2]])
        }
      }
      for (j1 in 1:nb) {
        for (j2 in 1:ns) {
          if (!is.na(qAr[[j2]][1])) {
            xrA <- rAA[[j2]] %*% t(xjb[[j1]])
            Hbs[j1, j2] <- Hbs[j1, j2] + sum(xrA * qAr[[j2]])
            Jbs[j1, j2] <- Jbs[j1, j2] -
              0.5 * sum(tris[[j2]] * cs_xSr[[j1]]) -
              0.5 * sum(cs_xSr[[j1]] * cs_rAr[[j2]])
          }
        }
      }
    }
    for (j1 in 1:nb) {
      for (j2 in 1:j1) {
        Hb[j1, j2] <- Hb[j2, j1]
        Jb[j1, j2] <- Jb[j2, j1]
      }
    }
    for (j1 in 1:ns) {
      for (j2 in 1:j1) {
        Hs[j1, j2] <- Hs[j2, j1]
        Js[j1, j2] <- Js[j2, j1]
      }
    }
  }
  H <- rbind(cbind(Hl, Hlb, Hls), cbind(t(Hlb), Hb, Hbs),
             cbind(t(Hls), t(Hbs), Hs))
  J <- rbind(cbind(Jl, Jlb, Jls), cbind(t(Jlb), Jb, Jbs),
             cbind(t(Jls), t(Jbs), Js))
  iII <- ginv(-H)
  iIIr <- iII %*% J %*% iII
  Vname <- c("lambda", paste("beta:", names(beta)),
             paste("alpha:", names(alp)))
  colnames(iII) <- Vname
  row.names(iII) <- Vname
  colnames(iIIr) <- Vname
  row.names(iIIr) <- Vname
  options(backup_options)
  structure(class = "bcmarg",
            list(call = Call, formula = formula, lambda = le, beta = beta,
                 alpha = alp, V = V, betainf = bcres, Vtheta.mod = iII,
                 Vtheta.rob = iIIr, logLik = lik, adj.prm = adj.prm,
                 glsObject = RS))
}

#' @export
logLik.bcmarg <- function(object, REML = FALSE, ...) {
  if (REML) {
    stop("REML method can not be used in bcmmrm.")
  }
  structure(object$logLik,
            df = 1 + length(c(object$beta, object$alpha)),
            class = "logLik")
}

#' @export
coef.bcmarg <- function(object, ...) {
  coef(object$glsObject)
}


#' @export
print.bcmarg <- function(x, digits = 3, ...) {
    mCall <- x$call
    covstr <- mCall$structure
    if (is.null(covstr) & !is.null(mCall$id)) {
      covstr <- "UN"
    }
    cat("Box-Cox transformed mixed model analysis\n")
    cat("  Formula:", deparse(x$formula), "\n")
    cat("  Time:", deparse(mCall$time), "\n")
    cat("  ID:", deparse(mCall$id), "\n")
    cat("  Covariance structure:", deparse(covstr), "\n")
    cat("  Data:", deparse(mCall$data), "\n")
    cat("  Estimated transformation parameter: ",
        format(x$lambda, digit = digits), "\n")
    cat("  Log-likelihood: ", format(x$logLik), "\n", sep = "")
    cat("\nCoefficients:\n")
    print(x$beta, digit = digits, ...)
    invisible(x)
  }

#' @export
summary.bcmarg <- function(object, ...) {
  structure(class = "summary.bcmarg", object)
}

#' @export
print.summary.bcmarg <- function(x, digits = 3, ...) {
    mCall <- x$call
    covstr <- mCall$structure
    if (is.null(covstr) & !is.null(mCall$id)) {
      covstr <- "UN"
    }
    V <- x$V
    Sd <- sqrt(diag(V))
    R <- V / (Sd %*% t(Sd))
    betainf <- x$betainf
    betainf[, 4] <- round(betainf[, 4], digits = digits)

    cat("Box-Cox transformed mixed model analysis\n")
    cat("  Formula:", deparse(x$formula), "\n")
    cat("  Time:", deparse(mCall$time), "\n")
    cat("  ID:", deparse(mCall$id), "\n")
    cat("  Covariance structure:", deparse(covstr), "\n")
    cat("  Data:", deparse(mCall$data), "\n")
    cat("  Log-likelihood: ", format(x$logLik), "\n", sep = "")
    cat("  Estimated transformation parameter: ",
        format(x$lambda, digits = digits), "\n")
    cat("\nCoefficients on the transformed scale:\n")
    print(betainf, digits = digits, ...)
    cat("\nCovariance parameters on the transformed scale:\n")
    print(x$alp, digits = digits)
    cat("\nCorrelations on the transformed scale:\n")
    print(R, digits = digits, ...)
    invisible(x)
}

#' @export
vcov.bcmarg <- function(object, robust = TRUE, ...) {
  if (robust) {
    vcov <- object$Vtheta.rob
  } else {
    vcov <- object$Vtheta.mod
  }
  return(vcov)
}


#' @export
fitted.bcmarg <- function(object, transformed = FALSE, ...) {
  z <- fitted(object$glsObject)
  if  (transformed) {
    fitted <- z
  } else {
    lmd <- object$lambda
    if (lmd == 0) {
      fitted <- exp(z)
    } else {
      fitted <- (lmd * z + 1) ^ (1 / lmd)
    }
  }
  return(fitted)
}


#' @export
predict.bcmarg <- function(object, newdata = NULL, transformed = FALSE, ...) {
  if (missing(newdata)) {
    z <- predict(object$glsObject)
  } else {
    z <- predict(object$glsObject, newdata)
  }
  if  (transformed) {
    pred<- z
  } else {
    lmd <- object$lambda
    if (lmd == 0) {
      pred<- exp(z)
    } else {
      pred <- (lmd * z + 1) ^ (1 / lmd)
    }
  }
  return(pred)
}

#' CD4 Cells Count Data in Aids Randomized Controlled Clinical Trial.
#'
#' The data are from a randomized, double-blind study of acquired immune
#' deficiency syndrome (AIDS) patients with advanced immune suppression
#' (cluster of differentiation 4 (CD4) cells count of less than or equal to 50
#' cells/mm3) (Henry et al., 1998; Fitzmaurice et al., 2011). Patients in the
#' AIDS Clinical Trial Group Study 193A were randomized to dual or triple
#' combinations of human immunodeficiency virus-1 reverse transcriptase
#' inhibitors. Specifically, patients were randomized to one of four daily
#' regimens.
#'
#' @name aidscd4
#'
#' @docType data
#'
#' @format A data frame with 4708 observations on the following 7 variables:
#'   \describe{
#'     \item{\code{id}}{patient identifier; in total there are 1177 patients.}
#'     \item{\code{weekc}}{nominal visit variable (weeks 8, 16, 24, 32).}
#'     \item{\code{treatment}}{allocated treatment regimens; 1 = zidovudine
#'           alternating monthly with 400mg didanosine, 2 = zidovudine plus
#'           2.25mg of zalcitabine, 3 = zidovudine plus 400mg of didanosine,
#'           and 4 = zidovudine plus 400mg of didanosine plus 400mg of
#'           nevirapine.}
#'     \item{\code{age}}{patients' age (years).}
#'     \item{\code{sex}}{patients' sex (1 = male, 0 = female)}
#'     \item{\code{cd4.bl}}{baseline value of CD4 cells count + 1.}
#'     \item{\code{cd4}}{CD4 cells count + 1.}
#'   }
#'
#' @examples data(aidscd4)
#'
#' @source \url{https://content.sph.harvard.edu/fitzmaur/ala2e/}
#'
#' @references \itemize{
#'   \item Henry, K., Erice, A., Tierney, C., Balfour, H.H. Jr., Fischl, M.A.,
#'         Kmack, A., Liou, S.H., Kenton, A., Hirsch, M.S., Phair, J.,
#'         Martinez, A., Kahn, J.O., for the AIDS Clinical Trial Group 193A
#'         Study Team. (1998). A randomized, controlled, double-blind study
#'         comparing the survival benefit of four different reverse
#'         transcriptase inhibitor therapies (three-drug, two-drug, and
#'         alternating drug) for the treatment of advanced AIDS.
#'         \emph{Journal of Acquired Immune Deficiency Syndromes and
#'         Human Retrovirology}, 19, 339-349,
#'         \doi{10.1097/00042560-199812010-00004}.
#'   \item Fitzmaurice, G.M., Laird, N.M., and Ware, J.H. (2011).
#'         \emph{Applied Longitudinal Analysis 2nd ed.}, Wiley, New York,
#'         \doi{10.1002/9781119513469}.
#'   }
NULL

#' Model Median Inference for Longitudinal Data in Randomized Clinical Trials.
#'
#' \code{bcmmrm} provides inference results for the model median differences
#' between treatment groups proposed by Maruo et al. (2017), which focuses on
#' continuous and positive longitudinally observed outcomes and a situation
#' where the efficacy of some treatments is compared based on a randomized,
#' parallel group clinical trial. If \code{time} and \code{id} are not
#' specified, inference results reduce to the results for the context of
#' linear regression model provided by Maruo et al. (2015).
#'
#' @param outcome a name of positive outcome (dependent) variable included in
#'   \code{data}.
#' @param group  a name of treatment group variable included in \code{data}.
#' @param data a data frame that may include \code{outcome}, \code{group},
#'   \code{time}, \code{id}, and specified covariate variables.
#' @param time a name of time variable for repeated measurements included
#'   in \code{data}. The default is \code{NULL}.
#' @param id a name of subject id variable for repeated measurements included
#'   in \code{data}. The default is \code{NULL}.
#' @param covv a character vector with names of covariate variables included
#'   in \code{data}. The default is \code{NULL}.
#' @param cfactor an integer vector including nominal variable indicators for
#'   covariate variables. Nominal variable: \code{1}, continuous variable:
#'   \code{0}. The default is \code{NULL}.
#' @param structure specify the covariance structure from \code{c("UN", "CS",
#'   "AR(1)")}. The default is \code{"UN"}.
#' @param conf.level a numeric value of the confidence level for the
#'   confidence intervals. The default is 0.95.
#' @param lmdint a vector containing the end-points of the interval to be
#'   searched for a transformation parameter. The default is \code{c(-3, 3)}.
#' @param glabel a vector of length number of treatment groups containing
#'   the labels of \code{group} variable. The default is \code{NULL} and the
#'   levels of \code{group} variable in \code{data} are used.
#' @param tlabel a vector of length number of repeated measures containing
#'   the labels of \code{time} variable. The default is \code{NULL} and the
#'   levels of \code{time} variable in \code{data} are used.
#' @param covint a logical value that specifies whether to include the
#'   interaction between time and covariate(s) in the model. The default is
#'   \code{FALSE}.
#'
#' @return an object of class "\code{bcmmrm}" representing the results of model
#'   median inference based on the Box-Cox transformed MMRM model.
#'   Generic functions such as \code{print}, \code{plot}, and \code{summary}
#'   have methods to show the results of the fit. See \code{\link{bcmmrmObject}}
#'   for the components of the fit.
#'
#' @note If baseline observation for the outcome variable is available, Box-Cox
#'   transformed baseline should be included as a covariate for accuracy of
#'   estimation.\cr Although this function can be applied to non-randomized
#'   trial data, performances of the above approach have not evaluated in
#'   context of non-randomized trials.
#'
#' @references \itemize{
#'   \item Maruo, K., Isogawa, N., Gosho, M. (2015). Inference of median
#'         difference based on the Box-Cox model in randomized clinical trials.
#'         \emph{Statistics in Medicine}, 34, 1634-1644,
#'         \doi{10.1002/sim.6408}.\cr
#'   \item Maruo, K., Yamaguchi, Y., Noma, H., Gosho, M. (2017). Interpretable
#'         inference on the mixed effect model with the Box-Cox transformation.
#'         \emph{Statistics in Medicine}, 36, 2420-2434,
#'         \doi{10.1002/sim.7279}.
#' }
#'
#' @seealso \code{\link{bcmarg}}, \code{\link{bcmmrmObject}}
#'
#' @examples
#' data(aidscd4)
#'
#' # covariate: Box-Cox transformed baseline (continuous) and sex (nominal),
#' # covariance structure: AR(1) structure
#' # *Note: The UN structure is preferred although the AR(1)
#' #        structure is used in this example to reduce calculation time
#'
#' # Box-Cox transformation for the baseline
#' aidscd4$cd4.bl.tr <- bct.v(aidscd4$cd4.bl)$transformed
#'
#' # Median inference for each group and week
#' bcmmrm(outcome = cd4, group = treatment, data = aidscd4, time = weekc,
#'        id = id, covv = c("cd4.bl.tr", "sex"), cfactor = c(0, 1),
#'        structure = "AR(1)", glabel = c("Zid/Did", "Zid+Zal", "Zid+Did",
#'        "Zid+Did+Nev"))
#'
#' @importFrom stats model.matrix pnorm pt qnorm qt
#'
#' @export
bcmmrm <- function(outcome, group, data, time = NULL, id = NULL,
                   covv = NULL, cfactor = NULL, structure = "UN",
                   conf.level = 0.95, lmdint = c(-3, 3),
                   glabel = NULL, tlabel = NULL, covint = FALSE) {

  backup_options <- options()
  if (conf.level <= 0 | conf.level >= 1) {
    stop("conf.level must be within the range of c(0,1)")
  }
  Call <- match.call()
  y <- deparse(substitute(outcome))
  data0 <- data
  group <- deparse(substitute(group))
  data0$group <- data0[, group]
  data0$y <- data0[, y]
  covform <- ""
  cct <- 0
  if (!is.null(covv)) {
    cct <- numeric(length(covv)) + 1
    for (i in seq_len(length(covv))) {
      if (cfactor[i] == 0) {
        if (!covint | i != 1){
          covform <- paste(covform, "+", covv[i])
        } else {
          covform <- covv[i]
        }
      } else {
        if (!covint | i != 1){
          covform <- paste(covform, "+as.factor(", covv[i], ")")
        } else {
          covform <- paste("as.factor(", covv[i], ")")
        }
        cct[i] <- length(unique(data0[, covv[i]])) - 1
      }
    }
  }
  ng <- length(unique(data0$group))
  if (is.null(glabel)) {
    glabel <- names(table(as.factor(data0$group)))
  }
  group.tbl <- data.frame(code = 1:ng, label = glabel)
  if (deparse(substitute(time)) != "NULL" &
      deparse(substitute(id)) != "NULL") {
    if (is.null(Call$structure)) {
      Call$structure <- "UN"
    }
    time <- deparse(substitute(time))
    id <- deparse(substitute(id))
    data0$time0 <- data0[, time]
    time.tbl <- sort(unique(data0$time0))
    for (i in seq_len(length(time.tbl))) {
      data0[data0$time0 == time.tbl[i], "time"] <- i
    }
    data0$id <- data0[, id]
    if (covint & !is.null(covv)){
      formula <- formula(paste("y ~ as.factor(group) * as.factor(time) +
                               as.factor(time) * (", covform, ")"))
    } else {
      formula <- formula(paste("y ~ as.factor(group) * as.factor(time)",
                               covform))
    }
    nt <- length(unique(data0$time))
    if (is.null(tlabel)) {
      tlabel <- time.tbl
    }
    time.tbl <- data.frame(code = 1:nt, label = tlabel)
    msflg <- table(data0$id, is.na(data0$y))[, 1]
    N <- sum(msflg != 0)
    omis <- names(msflg)[msflg == 0]
    if (length(omis) > 0L) {
      for (i in seq_len(length(omis))) {
        data0 <- data0[data0$id != omis[i], ]
      }
    }
    rm(id)
    rm(time)
    try1 <- bcmarg(formula, data0, time, id, structure, lmdint)
    fitted <- try1$glsObject$fitted
  } else {
    formula <- formula(paste("y ~ as.factor(group)", covform))
    data0$time <- 1
    nt <- 1
    try1 <- bcmarg(formula, data0)
    time.tbl <- data.frame(code = 1, label = 1)
    fitted <- try1$glsObject$fitted.values
  }

  le <- try1$lambda
  lik <- try1$logLik
  beta <- try1$beta
  alp <- try1$alpha
  V <- try1$V
  RR <- V / (sqrt(diag(V)) %*% t(sqrt(diag(V))))
  nc <- sum(cct)
  nb <- length(beta)
  ns <- length(alp)
  iII <- try1$Vtheta.mod
  iIIr <- try1$Vtheta.rob
  options(na.action = "na.pass")
  X <- model.matrix(formula, data = data0)
  if (nt != 1) {
    X <- X[!duplicated(data0$id), ]
  }
  dimnames(X) <- NULL
  xcm <- 0
  if (!is.null(covv)) {
    if (nc == 1)  {
      xcm <- mean(X[, ng + nt])
    } else {
      xcm <- c(colMeans(X[, (ng + nt):(ng + nt + nc - 1)]))
    }
  }
  adj.prm <- try1$adj.prm
  N <- adj.prm[1]
  Ncc <- adj.prm[2]
  n.data <- adj.prm[3]
  n.na <- adj.prm[4]
  if (structure == "UN" & nt != 1) {
    nu <- Ncc - nt
    nu2 <- sqrt(Ncc / nu)
  } else if (nt == 1) {
    nu <- n.data - nb
    nu2 <- sqrt(n.data / nu)
  } else{
    nu <- (N - ng) * (nt - 1) - n.na
    nu2 <- sqrt(n.data / (n.data - nb))
  }
  cbn <- choose(ng, 2)
  comb <- matrix(0, cbn, 2)
  count <- 1
  for (ii in 1:ng) {
    for (jj in 1:ng) {
      if (ii < jj) {
        comb[count, 1] <- ii
        comb[count, 2] <- jj
        count <- count + 1
      }
    }
  }
  median.mod <- list()
  median.rob <- list()
  median.mod.adj <- list()
  median.rob.adj <- list()
  meddif.mod <- list()
  meddif.rob <- list()
  meddif.mod.adj <- list()
  meddif.rob.adj <- list()
  for (tp0 in 1:nt) {
    xi <- numeric(ng)
    bt <- 0
    dbt <- numeric(nt - 1)
    if (tp0 != 1) {
      bt <- beta[ng + tp0 - 1]
      dbt[tp0 - 1] <- 1
    }
    bc <- 0
    bct <- 0
    if (!is.null(covv)) {
      bc <- beta[(ng + nt):(ng + nt + nc - 1)]
      if (covint) {
        bct0 <- beta[(ng + nt + nc + (nt - 1) * (ng - 1)):nb]
        if (tp0 == 1) {
          bct <- numeric(nc)
        } else {
          bct <- bct0[seq(tp0 - 1, (nt - 1) * (nc - 1) + (tp0 - 1),
                          by = (nt - 1))]
        }
        dbct <- numeric(length(bct0))
        for (ic in 1:nc) {
          dbct[bct0 == bct[ic]] <- xcm[ic]
        }
      }
    }
    dtv <- numeric(ns)
    SExi <- numeric(ng)
    SExir <- numeric(ng)
    Dt <- matrix(0, 1 + nb + ns, ng)
    for (i in 1:ng) {
      bg <- 0
      dbg <- numeric(ng - 1)
      if (i != 1) {
        bg <- beta[i]
        dbg[i - 1] <- 1
      }
      bgt <- 0
      dbgt <- numeric((ng - 1) * (nt - 1))
      if (i != 1 & tp0 != 1) {
        bgti <- (tp0 - 2) * (ng - 1) + i - 1
        bgt <- beta[ng + nt + nc - 1 + bgti]
        dbgt[bgti] <- 1
      }
      xi[i] <- (le * (beta[1] + bg + bt + bgt + sum(xcm * bc) + sum(xcm * bct))
                + 1) ^ (1 / le)
      dtl <- le ^ (-2) * xi[i] * (1 - le * log(xi[i]) - xi[i] ^ (-le))
      beta_loc <- numeric(nb)
      if (!is.null(covv)) {
        if (covint) {
          beta_loc <- c(1, dbg, dbt, xcm, dbgt, dbct)
        } else {
          beta_loc <- c(1, dbg, dbt, xcm, dbgt)
        }
      } else {
        beta_loc <-  c(1, dbg, dbt, dbgt)
      }
      dtb <- xi[i] ^ (1 - le) *  beta_loc
      Dt[, i] <- c(dtl, dtb, dtv)
      SExi[i] <- sqrt(c(t(Dt[, i]) %*% iII %*% Dt[, i]))
      SExir[i] <- sqrt(c(t(Dt[, i]) %*% iIIr %*% Dt[, i]))
    }
    delta <- numeric(cbn)
    SEd <- numeric(cbn)
    SEdr <- numeric(cbn)
    for (i in 1:cbn) {
      delta[i] <- xi[comb[i, 2]] - xi[comb[i, 1]]
      SEd[i] <- sqrt(c(t(Dt[, comb[i, 2]] - Dt[, comb[i, 1]]) %*% iII %*%
                         (Dt[, comb[i, 2]] - Dt[, comb[i, 1]])))
      SEdr[i] <- sqrt(c(t(Dt[, comb[i, 2]] - Dt[, comb[i, 1]]) %*% iIIr %*%
                          (Dt[, comb[i, 2]] - Dt[, comb[i, 1]])))
    }
    SExi2 <- SExi * nu2
    SExir2 <- SExir * nu2
    SEd2 <- SEd * nu2
    SEdr2 <- SEdr * nu2
    sig.level <- 1 - conf.level
    tt <- qnorm(1 - sig.level / 2)
    tvalue <- delta / SEd
    pvalue <- (1 - pnorm(abs(tvalue))) * 2
    tvaluer <- delta / SEdr
    pvaluer <- (1 - pnorm(abs(tvaluer))) * 2
    tvalue2 <- delta / SEd2
    tvaluer2 <- delta / SEdr2
    tt2 <- qt(1 - sig.level / 2, nu)
    pvalue2 <- (1 - pt(abs(tvalue2), nu)) * 2
    pvaluer2 <- (1 - pt(abs(tvaluer2), nu)) * 2
    lowerm <- xi - SExi * tt
    upperm <- xi + SExi * tt
    lowermr <- xi - SExir * tt
    uppermr <- xi + SExir * tt
    lowerm2 <- xi - SExi2 * tt2
    upperm2 <- xi + SExi2 * tt2
    lowermr2 <- xi - SExir2 * tt2
    uppermr2 <- xi + SExir2 * tt2
    lowerd <- delta - SEd * tt
    upperd <- delta + SEd * tt
    lowerdr <- delta - SEdr * tt
    upperdr <- delta + SEdr * tt
    lowerd2 <- delta - SEd2 * tt2
    upperd2 <- delta + SEd2 * tt2
    lowerdr2 <- delta - SEdr2 * tt2
    upperdr2 <- delta + SEdr2 * tt2
    median.mod[[tp0]] <- data.frame(group = 1:ng, median = xi, SE = SExi,
                                    lower.CL = lowerm, upper.CL = upperm)
    median.rob[[tp0]] <- data.frame(group = 1:ng, median = xi, SE = SExir,
                                    lower.CL = lowermr, upper.CL = uppermr)
    median.mod.adj[[tp0]] <- data.frame(group = 1:ng, median = xi, SE = SExi2,
                                        lower.CL = lowerm2, upper.CL = upperm2)
    median.rob.adj[[tp0]] <- data.frame(group = 1:ng, median = xi, SE = SExir2,
                                        lower.CL = lowermr2,
                                        upper.CL = uppermr2)
    meddif.mod[[tp0]] <- data.frame(group1 = comb[, 2], group0 = comb[, 1],
                                    delta = delta, SE = SEd, lower.CL = lowerd,
                                    upper.CL = upperd, t.value = tvalue,
                                    p.value = pvalue)
    meddif.rob[[tp0]] <- data.frame(group1 = comb[, 2], group0 = comb[, 1],
                                    delta = delta, SE = SEdr,
                                    lower.CL = lowerdr, upper.CL = upperdr,
                                    t.value = tvaluer, p.value = pvaluer)
    meddif.mod.adj[[tp0]] <- data.frame(group1 = comb[, 2], group0 = comb[, 1],
                                        delta = delta, SE = SEd2,
                                        lower.CL = lowerd2, upper.CL = upperd2,
                                        t.value = tvalue2, p.value = pvalue2)
    meddif.rob.adj[[tp0]] <- data.frame(group1 = comb[, 2], group0 = comb[, 1],
                                        delta = delta, SE = SEdr2,
                                        lower.CL = lowerdr2,
                                        upper.CL = upperdr2,
                                        t.value = tvaluer2, p.value = pvaluer2)
  }
  data$ytr <- (data0$y ^ le - 1) / le
  data$ytr.fitted <- data$ytr
  data$ytr.fitted[!is.na(data$ytr.fitted)] <- fitted
  data$res.tr <- data$ytr - data$ytr.fitted
  options(backup_options)
  structure(class = "bcmmrm",
            list(call = Call, median.mod = median.mod,
                 median.rob = median.rob, median.mod.adj = median.mod.adj,
                 median.rob.adj = median.rob.adj,
                 meddif.mod = meddif.mod, meddif.rob = meddif.rob,
                 meddif.mod.adj = meddif.mod.adj,
                 meddif.rob.adj = meddif.rob.adj, lambda = le, R = RR,
                 logLik = lik, betainf = try1$betainf, time.tbl = time.tbl,
                 group.tbl = group.tbl, inf.marg = try1, outdata = data,
                 conf.level = conf.level))
}


#' @export
logLik.bcmmrm <- function(object, REML = FALSE, ...) {
  if (REML) {
    stop("REML method can not be used in bcmmrm.")
  }
  structure(object$logLik,
            df = 1 + length(c(object$inf.marg$beta, object$inf.marg$alpha)),
            class = "logLik")
}

#' @export
coef.bcmmrm <- function(object, ...) {
  coef(object$inf.marg$glsObject)
}

#' @export
print.bcmmrm <- function(x, ...) {
    mCall <- x$call
    covstr <- mCall$structure
    if (is.null(covstr) & !is.null(mCall$id)) {
      covstr <- "UN"
    }
    cat("Model median estimation based on MMRM with Box-Cox transformation\n")
    cat("  Outcome:", deparse(mCall$outcome), "\n")
    cat("  Group:", deparse(mCall$group), "\n")
    cat("  Time:", deparse(mCall$time), "\n")
    cat("  ID:", deparse(mCall$id), "\n")
    cat("  Covariate(s):", deparse(mCall$covv), "\n")
    cat("  Covariance structure:", deparse(covstr), "\n")
    cat("  Data:", deparse(mCall$data), "\n")
    cat("  Estimated transformation parameter: ",
        format(x$lambda, digits = 3), "\n")
    cat("  Log-likelihood: ", format(x$logLik), "\n", sep = "")

    nt <- length(x$median.mod)
    med <- c()
    for (i in 1:nt) {
      med <- cbind(med, x$median.mod[[i]]$median)
    }
    med <- cbind(x$group.tbl$label, as.data.frame(med))
    if (nt > 1) {
      names(med) <- c(paste(mCall$group, "|", mCall$time), x$time.tbl$label)
      cat("\nModel median estimates (row: group, col: time):\n")
      print(med, digits = 3, ...)
    } else {
      names(med) <- c(paste(mCall$group), "Estimate")
      cat("\nModel median estimates:\n")
      print(med, digits = 3, ...)
    }
    invisible(x)
}

#' @export
vcov.bcmmrm <- function(object, robust = TRUE, ...) {
  if (robust) {
    vcov <- object$inf.marg$Vtheta.rob
  } else {
    vcov <- object$inf.marg$Vtheta.mod
  }
  return(vcov)
}

#' @export
fitted.bcmmrm <- function(object, transformed = FALSE, ...) {
  z <- object$outdata$ytr.fitted
  if  (transformed) {
    fitted <- z
  } else {
    lmd <- object$lambda
    if (lmd == 0) {
      fitted <- exp(z)
    } else {
      fitted <- (lmd * z + 1) ^ (1 / lmd)
    }
  }
  return(fitted)
}

library(testthat)
library(bcmixed)

context("Test bcmmrm")
data(aidscd4)
se1 <- bcmmrm(cd4, treatment, aidscd4, weekc, id, c("cd4.bl", "age", "sex"),
              c(0, 0, 1), "AR(1)")$meddif.rob.adj[[4]][2, 4]
se2 <- bcmmrm(cd4, treatment,
              aidscd4[aidscd4$weekc == 32, ])$meddif.rob.adj[[1]][2, 4]
test_that("Test bcmmrm not cleared", {
  expect_that(round(se1, 3), equals(1.292))
  expect_that(round(se2, 3), equals(1.718))
})

#' Box-whisker plot for a bcmmrm Object.
#'
#' Box-whisker plot for the transformed residuals of each treatment groups
#' at a specified time point with error bar plot (mean +- SD)  using
#' \code{bcmmrmObject}.
#'
#' @param x an object inheriting from class "\code{bcmmrm}", representing
#' the Box-Cox transformed MMRM analysis.
#' @param timepoint an numeric value of a specified level of \code{time}
#' variable. The default is \code{NULL} and the last level is specified.
#' @param xlab a title for the x axis. The default is \code{NULL} and
#'  the name of \code{time} or \code{group} variable is used.
#' @param ylab a title for the y axis. The default is \code{NULL} and
#'  the default title is "Transformed residuals".
#' @param main a main title for the plot. The default is \code{TRUE} and
#'  default title is "Box-whisker plot for transformed residuals".
#' @param sub a sub title for the plot. The default is \code{NULL}.
#' @param  verbose a logical optional value specifying to print the detailed
#' plot information in the console. The default is \code{FALSE}.
#' @param ... some methods for this generic require additional arguments.
#'
#' @return a box-whisker plot for transformed residual.
#'
#' @seealso \code{\link{bcmmrm}}, \code{\link{bcmmrmObject}}
#'
#' @examples
#'  data(aidscd4)
#'  lmd.bl <- bcreg(cd4.bl ~ 1, data = aidscd4[aidscd4$weekc == 8, ])$lambda
#'  aidscd4$cd4.bl.tr <- (aidscd4$cd4.bl ^ lmd.bl - 1) / lmd.bl
#'  resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
#'                 time = weekc, id = id, covv = c("cd4.bl.tr", "sex"),
#'                 cfactor = c(0, 1), structure = "AR(1)", glabel =
#'                 c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
#'  boxplot(resar, xlab = "Treatment", col = 1:4)
#' @importFrom graphics boxplot axis abline points arrows lines
#' @importFrom stats sd
#' @export
boxplot.bcmmrm <- function(x, timepoint = NULL, xlab = NULL,
                           ylab = NULL, main = TRUE, sub = NULL,
                           verbose = FALSE, ...) {
  if (is.null(timepoint)) {
    nt <- length(x$median.mod)
    timepoint <- x$time.tbl$label[nt]
  }
  dgdat <- x$outdata[x$outdata[, deparse(x$call$time)] ==
                        timepoint, ]
  rgm <- ceiling(max(abs(dgdat$res.tr), na.rm = TRUE))
  group <- deparse(x$call$group)
  if (is.null(xlab)) {
    xlab <- group
  }
  if (is.null(ylab)) {
    ylab <- "Transformed residual"
  }
  if (!is.null(main)) {
    if (main) {
      main <- paste("Box-whisker plot for transformed residuals at",
                    deparse(x$call$time), "=", timepoint)
    }
  }
  bp <- boxplot(dgdat$res.tr ~ dgdat[, group],
          ylim = c(-rgm, rgm), main = main, sub = sub,
          ylab = ylab, xlab = xlab, xaxt = "n")
  glabel <- x$group.tbl$label
  axis(1, at = x$group.tbl$code, labels = glabel)
  abline(h = 0)
  xi <- 0.2 + seq(bp$n)
  mn.t <- tapply(dgdat$res.tr, dgdat[, group], mean, na.rm = TRUE)
  sd.t <- tapply(dgdat$res.tr, dgdat[, group], sd, na.rm = TRUE)
  points(xi, mn.t, pch = 4)
  arrows(xi, mn.t - sd.t, xi, mn.t + sd.t,
         code = 3, angle = 75, length = .1)
  if (verbose) {
    cat("Plot information:\n")
    cat("\nError bar plot: mean +- SD\n")
    cat("\nTimepoint:", deparse(x$call$time), "=", timepoint)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bct.v.R
\name{bct.v}
\alias{bct.v}
\title{Applying the Box-Cox Transformation to a numeric vector.}
\usage{
bct.v(y, lmdint = c(-3, 3))
}
\arguments{
\item{y}{a positive real number vector.}

\item{lmdint}{a vector containing the end-points of the interval to be
searched for a transformation parameter. Default is c(-3, 3).}
}
\value{
\code{bct.v} returns the Box-Cox transformed numeric vector and
estimated transformation parameter.
\describe{
  \item{\code{transformed}}{The Box-Cox transformed numeric vector.}
  \item{\code{lambda}}{a numeric value of the estimate of the transformation
        parameter.}
}
}
\description{
\code{bct.v} returns the Box-Cox transformed numeric vector (Box and Cox,
1964).
}
\examples{
  y <- exp(rnorm(50))
  bct.v(y)

}
\references{
Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
(with discussion).
\emph{Journals of the Royal Statistical Society, Series B}, 26,
211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bct.R
\name{bct}
\alias{bct}
\title{Applying the Box-Cox Transformation.}
\usage{
bct(y, lambda)
}
\arguments{
\item{y}{a positive real number vector.}

\item{lambda}{a scalar transformation parameter.}
}
\value{
bct returns the Box-Cox transformed numeric vector,
\code{z = log(y)} for \code{lambda = 0},
\code{z = (y ^ lambda - 1) / lambda} for \code{lambda ne 1}.
}
\description{
\code{bct} returns the Box-Cox transformed numeric vector (Box and Cox,
1964).
}
\examples{
  y <- exp(rnorm(10))
  z <- bct(y, 0) #log transformation

}
\references{
Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
(with discussion).
\emph{Journals of the Royal Statistical Society, Series B}, 26,
211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcmixed-package.R
\docType{package}
\name{bcmixed-package}
\alias{bcmixed}
\alias{bcmixed-package}
\title{bcmixed: Mixed Effect Model with the Box-Cox Transformation}
\description{
Inference on the marginal model of the mixed effect model with the Box-Cox transformation and on the model median differences between treatment groups for longitudinal randomized clinical trials. These statistical methods are proposed by Maruo et al. (2017) \doi{10.1002/sim.7279}.
}
\author{
\strong{Maintainer}: Kazushi Maruo \email{kazushi.maruo@gmail.com}

Authors:
\itemize{
  \item Kazushi Maruo \email{kazushi.maruo@gmail.com}
  \item Ryota Ishii
}

Other contributors:
\itemize{
  \item Yusuke Yamaguchi [contributor]
  \item Masahiko Gosho [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aidscd4.R
\docType{data}
\name{aidscd4}
\alias{aidscd4}
\title{CD4 Cells Count Data in Aids Randomized Controlled Clinical Trial.}
\format{
A data frame with 4708 observations on the following 7 variables:
  \describe{
    \item{\code{id}}{patient identifier; in total there are 1177 patients.}
    \item{\code{weekc}}{nominal visit variable (weeks 8, 16, 24, 32).}
    \item{\code{treatment}}{allocated treatment regimens; 1 = zidovudine
          alternating monthly with 400mg didanosine, 2 = zidovudine plus
          2.25mg of zalcitabine, 3 = zidovudine plus 400mg of didanosine,
          and 4 = zidovudine plus 400mg of didanosine plus 400mg of
          nevirapine.}
    \item{\code{age}}{patients' age (years).}
    \item{\code{sex}}{patients' sex (1 = male, 0 = female)}
    \item{\code{cd4.bl}}{baseline value of CD4 cells count + 1.}
    \item{\code{cd4}}{CD4 cells count + 1.}
  }
}
\source{
\url{https://content.sph.harvard.edu/fitzmaur/ala2e/}
}
\description{
The data are from a randomized, double-blind study of acquired immune
deficiency syndrome (AIDS) patients with advanced immune suppression
(cluster of differentiation 4 (CD4) cells count of less than or equal to 50
cells/mm3) (Henry et al., 1998; Fitzmaurice et al., 2011). Patients in the
AIDS Clinical Trial Group Study 193A were randomized to dual or triple
combinations of human immunodeficiency virus-1 reverse transcriptase
inhibitors. Specifically, patients were randomized to one of four daily
regimens.
}
\examples{
data(aidscd4)

}
\references{
\itemize{
  \item Henry, K., Erice, A., Tierney, C., Balfour, H.H. Jr., Fischl, M.A.,
        Kmack, A., Liou, S.H., Kenton, A., Hirsch, M.S., Phair, J.,
        Martinez, A., Kahn, J.O., for the AIDS Clinical Trial Group 193A
        Study Team. (1998). A randomized, controlled, double-blind study
        comparing the survival benefit of four different reverse
        transcriptase inhibitor therapies (three-drug, two-drug, and
        alternating drug) for the treatment of advanced AIDS.
        \emph{Journal of Acquired Immune Deficiency Syndromes and
        Human Retrovirology}, 19, 339-349,
        \doi{10.1097/00042560-199812010-00004}.
  \item Fitzmaurice, G.M., Laird, N.M., and Ware, J.H. (2011).
        \emph{Applied Longitudinal Analysis 2nd ed.}, Wiley, New York,
        \doi{10.1002/9781119513469}.
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcmarg.R
\name{bcmarg}
\alias{bcmarg}
\title{Marginal Model of the Mixed Effect Model with the Box-Cox Transformation.}
\usage{
bcmarg(
  formula,
  data,
  time = NULL,
  id = NULL,
  structure = "UN",
  lmdint = c(-3, 3)
)
}
\arguments{
\item{formula}{a two-sided linear formula object describing the model, with
the response on the left of a ~ operator and the terms, separated by +
operators, on the right.}

\item{data}{a data frame containing the variables used in the model.}

\item{time}{time variable name for repeated measurements. The default is
NULL.}

\item{id}{subject id variable name for repeated measurements. The default is
NULL.}

\item{structure}{specify the covariance structure from c("UN", "CS",
"AR(1)"). The default is "UN".}

\item{lmdint}{a vector containing the end-points of the interval to be
searched for a transformation parameter. The default is c(-3, 3).}
}
\value{
an object of class "\code{bcmarg}". Objects of this class have
methods for the generic functions  \code{coef}, \code{logLik},
\code{print}, and \code{summary}. The object includes following components
for the marginal model parameter inference:
\describe{
  \item{\code{lambda}}{a numeric value of the estimate of the transformation
        parameter.}
  \item{\code{beta}}{a vector with the estimates of the regression
        parameters.}
  \item{\code{alpha}}{a vector with the estimates of the covariance
        parameters.}
  \item{\code{V}}{variance-covariance matrix for any subject with no missing
        values.}
  \item{\code{betainf}}{a matrix containing the inference results for
       \code{beta} under the assumption that lambda is known.
       Note that standard errors might be underestimated
       although statistical tests would be asymptotically valid.}
  \item{\code{Vtheta.mod}}{model-based variance-covariance matrix for MLE of
        the vector of all parameters: \code{c(lambda, beta, alpha)}.}
  \item{\code{Vtheta.rob}}{robust variance-covariance matrix for MLE of
        the vector of all parameters.}
  \item{\code{logLik}}{a numeric value of the maximized likelihood.}
  \item{\code{adj.prm}}{a vector with parameters used for the empirical
        small sample adjustment in \code{\link{bcmmrm}}:
        c(number of subjects, number of completed subjects, number of
        outcome observations, number of missing observations).}
  \item{\code{glsObject}}{an object of "\code{gls}" (or "\code{lm}" when
        \code{time} and \code{id} are not specified) containing results of
        \code{gls} (or \code{lm}) function on the transformed scale. }
}
}
\description{
\code{bcmarg} returns the inference results the parameters of
  the marginal model of the linear mixed effect model with the Box-Cox
  transformation proposed by Maruo et al. (2017). If time and id are not
  specified, inference results reduce to the results for the context of
  linear regression model provided by Maruo et al. (2015).
}
\examples{
 data(aidscd4)
 bcmarg(cd4 ~ as.factor(treatment) * as.factor(weekc) + age,
        data = aidscd4, time = weekc, id = id, structure = "AR(1)")

}
\references{
\itemize{
  \item Maruo, K., Isogawa, N., Gosho, M. (2015). Inference of median
        difference based on the Box-Cox model in randomized clinical trials.
        \emph{Statistics in Medicine}, 34, 1634-1644,
        \doi{10.1002/sim.6408}.\cr
  \item Maruo, K., Yamaguchi, Y., Noma, H., Gosho, M. (2017). Interpretable
        inference on the mixed effect model with the Box-Cox transformation.
        \emph{Statistics in Medicine}, 36, 2420-2434,
        \doi{10.1002/sim.7279}.
}
}
\seealso{
\code{\link{bcmmrm}} \code{\link[nlme]{gls}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcreg.R
\name{bcreg}
\alias{bcreg}
\title{Linear regression model with the Box-Cox Transformation.}
\usage{
bcreg(formula, data, lmdint = c(-3, 3))
}
\arguments{
\item{formula}{a two-sided linear formula object describing the model, with
the response on the left of a ~ operator and the terms, separated by +
operators, on the right.}

\item{data}{a data frame in which to interpret the variables named in the
formula.}

\item{lmdint}{a vector containing the end-points of the interval to be
searched for a transformation parameter. Default is c(-3, 3).}
}
\value{
bcreg returns a list including following components:
\describe{
  \item{\code{lambda}}{a numeric value with the estimate of
        the transformation parameter.}
  \item{\code{beta}}{a vector with the estimates of the regression
        parameters.}
  \item{\code{sigma}}{a numeric value with the estimate of the scale
        parameter.}
  \item{\code{betainf}}{a data frame with inference results for \code{beta}
        under the assumption that \code{lambda} is known.}
  \item{\code{lik}}{a numeric value with the maximized likelihood.}
  \item{\code{lmObject}}{an object of "\code{lm}" containing the results of
        \code{lm} function on the transformed scale}
}
}
\description{
\code{bcreg} returns the maximum likelihood estimates for parameters of
the linear regression models with the Box-Cox transformation (Box and Cox,
1964).
}
\examples{
  data(aidscd4)
  #Transformation of baseline observation for aid.cd4 data
  bcreg(cd4.bl ~ 1, aidscd4[aidscd4$weekc == 8, ])

}
\references{
Box, G.E.P. and Cox, D.R. (1964). An analysis of transformations
(with discussion).
\emph{Journals of the Royal Statistical Society, Series B}, 26,
211-246, \doi{10.1111/j.2517-6161.1964.tb00553.x}.
}
\seealso{
\code{\link{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.bcmmrm.R
\name{summary.bcmmrm}
\alias{summary.bcmmrm}
\title{Summarize a bcmmrm Object.}
\usage{
\method{summary}{bcmmrm}(object, robust = TRUE, ssadjust = TRUE, ...)
}
\arguments{
\item{object}{an object inheriting from class "\code{bcmmrm}", representing
the Box-Cox transformed MMRM analysis.}

\item{robust}{an optional logical value used to specify whether to apply
the robust inference. The default is \code{TRUE}.}

\item{ssadjust}{an optional logical value used to specify whether to apply
the empirical small sample adjustment. The default is \code{TRUE}.}

\item{...}{some methods for this generic require additional arguments.
None are used in this method.}
}
\value{
an object inheriting from class \code{summary.bcmmrm} with all
components included in \code{object} (see \code{\link[nlme]{glsObject}} for
a full description of the components) plus the following components:
\describe{
  \item{\code{median}}{a list including inference results of the model median
        for specified values of \code{robust} and \code{ssadjust}.}
  \item{\code{meddif}}{a list including inference results of the model median
        difference for specified values of \code{robust} and
        \code{ssadjust}.}
  \item{\code{robust}}{a specified value of \code{robust}.}
  \item{\code{ssadjust}}{a specified value of \code{ssadjust}.}
}
}
\description{
Additional information about the Box-Cox transformed MMRM analysis
represented by \code{object} is extracted and included as components
of \code{object}.
}
\examples{
 data(aidscd4)
 resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
                 time = weekc, id = id, structure = "AR(1)",
                 glabel = c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
 summary(resar)

}
\seealso{
\code{\link{bcmmrm}}, \code{\link{bcmmrmObject}},
         \code{\link{summary}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcmmrm.R
\name{bcmmrm}
\alias{bcmmrm}
\title{Model Median Inference for Longitudinal Data in Randomized Clinical Trials.}
\usage{
bcmmrm(
  outcome,
  group,
  data,
  time = NULL,
  id = NULL,
  covv = NULL,
  cfactor = NULL,
  structure = "UN",
  conf.level = 0.95,
  lmdint = c(-3, 3),
  glabel = NULL,
  tlabel = NULL,
  covint = FALSE
)
}
\arguments{
\item{outcome}{a name of positive outcome (dependent) variable included in
\code{data}.}

\item{group}{a name of treatment group variable included in \code{data}.}

\item{data}{a data frame that may include \code{outcome}, \code{group},
\code{time}, \code{id}, and specified covariate variables.}

\item{time}{a name of time variable for repeated measurements included
in \code{data}. The default is \code{NULL}.}

\item{id}{a name of subject id variable for repeated measurements included
in \code{data}. The default is \code{NULL}.}

\item{covv}{a character vector with names of covariate variables included
in \code{data}. The default is \code{NULL}.}

\item{cfactor}{an integer vector including nominal variable indicators for
covariate variables. Nominal variable: \code{1}, continuous variable:
\code{0}. The default is \code{NULL}.}

\item{structure}{specify the covariance structure from \code{c("UN", "CS",
"AR(1)")}. The default is \code{"UN"}.}

\item{conf.level}{a numeric value of the confidence level for the
confidence intervals. The default is 0.95.}

\item{lmdint}{a vector containing the end-points of the interval to be
searched for a transformation parameter. The default is \code{c(-3, 3)}.}

\item{glabel}{a vector of length number of treatment groups containing
the labels of \code{group} variable. The default is \code{NULL} and the
levels of \code{group} variable in \code{data} are used.}

\item{tlabel}{a vector of length number of repeated measures containing
the labels of \code{time} variable. The default is \code{NULL} and the
levels of \code{time} variable in \code{data} are used.}

\item{covint}{a logical value that specifies whether to include the
interaction between time and covariate(s) in the model. The default is
\code{FALSE}.}
}
\value{
an object of class "\code{bcmmrm}" representing the results of model
  median inference based on the Box-Cox transformed MMRM model.
  Generic functions such as \code{print}, \code{plot}, and \code{summary}
  have methods to show the results of the fit. See \code{\link{bcmmrmObject}}
  for the components of the fit.
}
\description{
\code{bcmmrm} provides inference results for the model median differences
between treatment groups proposed by Maruo et al. (2017), which focuses on
continuous and positive longitudinally observed outcomes and a situation
where the efficacy of some treatments is compared based on a randomized,
parallel group clinical trial. If \code{time} and \code{id} are not
specified, inference results reduce to the results for the context of
linear regression model provided by Maruo et al. (2015).
}
\note{
If baseline observation for the outcome variable is available, Box-Cox
  transformed baseline should be included as a covariate for accuracy of
  estimation.\cr Although this function can be applied to non-randomized
  trial data, performances of the above approach have not evaluated in
  context of non-randomized trials.
}
\examples{
data(aidscd4)

# covariate: Box-Cox transformed baseline (continuous) and sex (nominal),
# covariance structure: AR(1) structure
# *Note: The UN structure is preferred although the AR(1)
#        structure is used in this example to reduce calculation time

# Box-Cox transformation for the baseline
aidscd4$cd4.bl.tr <- bct.v(aidscd4$cd4.bl)$transformed

# Median inference for each group and week
bcmmrm(outcome = cd4, group = treatment, data = aidscd4, time = weekc,
       id = id, covv = c("cd4.bl.tr", "sex"), cfactor = c(0, 1),
       structure = "AR(1)", glabel = c("Zid/Did", "Zid+Zal", "Zid+Did",
       "Zid+Did+Nev"))

}
\references{
\itemize{
  \item Maruo, K., Isogawa, N., Gosho, M. (2015). Inference of median
        difference based on the Box-Cox model in randomized clinical trials.
        \emph{Statistics in Medicine}, 34, 1634-1644,
        \doi{10.1002/sim.6408}.\cr
  \item Maruo, K., Yamaguchi, Y., Noma, H., Gosho, M. (2017). Interpretable
        inference on the mixed effect model with the Box-Cox transformation.
        \emph{Statistics in Medicine}, 36, 2420-2434,
        \doi{10.1002/sim.7279}.
}
}
\seealso{
\code{\link{bcmarg}}, \code{\link{bcmmrmObject}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.bcmmrm.R
\name{plot.bcmmrm}
\alias{plot.bcmmrm}
\title{Plot a bcmmrm Object.}
\usage{
\method{plot}{bcmmrm}(
  x,
  robust = TRUE,
  ssadjust = TRUE,
  dt = 1,
  timepoint = NULL,
  tnom = TRUE,
  xlab = NULL,
  ylab = NULL,
  xlim = NULL,
  ylim = NULL,
  lwd = 2,
  col = NULL,
  lty = NULL,
  main = TRUE,
  sub = NULL,
  legend = TRUE,
  loc = "topright",
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{an object inheriting from class "\code{bcmmrm}", representing
the Box-Cox transformed MMRM analysis.}

\item{robust}{an optional logical value used to specify whether to apply
the robust inference. The default is \code{TRUE}.}

\item{ssadjust}{an optional logical value used to specify whether to apply
the empirical small sample adjustment. The default is \code{TRUE}.}

\item{dt}{an numeric value indicating shift length between groups
in the longitudinal median plot. A multiplying factor for the default
settings specified (e.g. if \code{2} is specified,
shift length is twice longer than that for the default setting).
The default is \code{1}.}

\item{timepoint}{an numeric value of a specified level of \code{time}
variable at which median plot is created. When \code{timepoint} is
\code{NULL} and number of time points is not 1, longitudinal median plot
(x axis is \code{time}) is created. Otherwise, median plot where
x axis is \code{group} is created. The default is \code{NULL}.}

\item{tnom}{a optional logical value indicating the scale of x axis
of the longitudinal median plot. When \code{tnom} is \code{TRUE}, nominal
scale is used and widths between any combinations of neighbor time points
are same. When \code{tnom} is \code{FALSE}, actual scale of \code{time}
variable is used. The default is \code{TRUE}.}

\item{xlab}{a title for the x axis. The default is \code{NULL} and
the name of \code{time} or \code{group} variable is used.}

\item{ylab}{a title for the y axis. The default is \code{NULL} and
the name of \code{outcome} variable is used.}

\item{xlim}{a numeric vector with length of 2 indicating limits of x
axis. The default is \code{NULL} and limits are calculated automatically.}

\item{ylim}{a numeric vector with length of 2 indicating limits of y
axis. The default is \code{NULL} and limits are calculated automatically.}

\item{lwd}{an optional positive numeric value indicating line width.
The default is \code{2}.}

\item{col}{an integer or a character vector with length of the number of
groups indicating colors of lines for each treatment group. The default is
\code{NULL} and all of colors are black.}

\item{lty}{an optional integer or a character vector with length of the
number of groups indicating line types of lines for each treatment group.
The default is \code{NULL} and \code{1:ng} is used, where ng is number of
groups.}

\item{main}{a main title for the plot. The default is \code{TRUE} and
default title is "(Longitudinal) Plot for median of each group".}

\item{sub}{a sub title for the plot. The default is \code{NULL}.}

\item{legend}{a logical optional value specifying to add legends to
plots. When \code{legend} is \code{TRUE} legends are added to the plot.
Otherwise, legends are not added.  The default is \code{TRUE}.}

\item{loc}{a character value indicating the location of the legends.
The location can be specified by setting \code{loc} to a single keyword
from the list \code{"bottomright"}, \code{"bottom"}, \code{"bottomleft"},
\code{"left"}, \code{"topleft"}, \code{"top"}, \code{"topright"},
\code{"right"}, and \code{"center"}. The default is \code{"topright"}.}

\item{verbose}{a logical optional value specifying to print the detailed
analysis information in the console. The default is \code{FALSE}.}

\item{...}{some methods for this generic require additional arguments.}
}
\value{
a median plot.
}
\description{
Plot for the model medians of each treatment groups with the 95 percent
confidence intervals stored in \code{bcmmrmObject}.
}
\examples{
 data(aidscd4)
 resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
                time = weekc, id = id, structure = "AR(1)", glabel =
                c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
 plot(resar, xlab = "Week", ylab = "CD4+1", col = 1:4, main = NULL)
 plot(resar, timepoint = 32, xlab = "Treatment", ylab = "CD4+1")
}
\seealso{
\code{\link{bcmmrm}}, \code{\link{bcmmrmObject}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcmmrmObject.R
\name{bcmmrmObject}
\alias{bcmmrmObject}
\title{Fitted bcmmrm Object}
\value{
The following components must be included in a legitimate "bcmmrm"
object.\describe{
    \item{\code{call}}{a list containing an image of the \code{bcmmrm}
          call that produced the object.}
    \item{\code{median.mod}, \code{median.rob}, \code{median.mod.adj},
          \code{median.rob.adj}}{lists including inference results for the
          model medians for all groups. Levels of the list are time points,
          where correspondence table is given as \code{time.tbl$code}.
          mod: model-based inference, rob: robust inference,
          adj: with empirical small sample adjustment.}
    \item{\code{meddif.mod}, \code{meddif.rob}, \code{meddif.mod.adj},
          \code{meddif.rob.adj}}{lists including inference results for the
          for the model median differences between all pairs of groups
          (group1 - group0). Levels of the list are time points,
          where correspondence table is given as \code{time.tbl$code}.
          mod: model-based inference, rob: robust inference,
          adj: with empirical small sample adjustment.}
    \item{\code{lambda}}{a numeric value of estimates of the transformation
          parameter.}
    \item{\code{R}}{a correlation matrix for transformed outcomes.}
    \item{\code{betainf}}{inference results for beta under the assumption
          that lambda is known. Note that standard errors might be
          underestimated although statistical tests would be asymptotically
          valid.}
    \item{\code{time.tbl}}{a data frame of a correspondence table for
          the timepoints.}
    \item{\code{group.tbl}}{a data frame of a correspondence table for
          treatment groups.}
    \item{\code{inf.marg}}{a list with results of \code{\link{bcmarg}}
          function.}
    \item{\code{outdata}}{a data frame where the transformed outcome
          (\code{ytr}), the fitted values on the transformed scale
          (\code{ytr.fitted}), and the residuals on the transformed scale
          (\code{res.tr}) are added to input data.}
    \item{\code{conf.level}}{a numeric value of the specified confidence
          level.}
  }
}
\description{
An object returned by the \code{\link{bcmmrm}} function, inheriting from
class "bcmmrm" and representing the Box-Cox transformed MMRM analysis.
Objects of this class have methods for the generic functions \code{boxplot},
\code{coef}, \code{logLik}, \code{plot}, \code{print}, and \code{summary}.
}
\seealso{
\code{\link{bcmmrm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxplot.bcmmrm.R
\name{boxplot.bcmmrm}
\alias{boxplot.bcmmrm}
\title{Box-whisker plot for a bcmmrm Object.}
\usage{
\method{boxplot}{bcmmrm}(
  x,
  timepoint = NULL,
  xlab = NULL,
  ylab = NULL,
  main = TRUE,
  sub = NULL,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{an object inheriting from class "\code{bcmmrm}", representing
the Box-Cox transformed MMRM analysis.}

\item{timepoint}{an numeric value of a specified level of \code{time}
variable. The default is \code{NULL} and the last level is specified.}

\item{xlab}{a title for the x axis. The default is \code{NULL} and
the name of \code{time} or \code{group} variable is used.}

\item{ylab}{a title for the y axis. The default is \code{NULL} and
the default title is "Transformed residuals".}

\item{main}{a main title for the plot. The default is \code{TRUE} and
default title is "Box-whisker plot for transformed residuals".}

\item{sub}{a sub title for the plot. The default is \code{NULL}.}

\item{verbose}{a logical optional value specifying to print the detailed
plot information in the console. The default is \code{FALSE}.}

\item{...}{some methods for this generic require additional arguments.}
}
\value{
a box-whisker plot for transformed residual.
}
\description{
Box-whisker plot for the transformed residuals of each treatment groups
at a specified time point with error bar plot (mean +- SD)  using
\code{bcmmrmObject}.
}
\examples{
 data(aidscd4)
 lmd.bl <- bcreg(cd4.bl ~ 1, data = aidscd4[aidscd4$weekc == 8, ])$lambda
 aidscd4$cd4.bl.tr <- (aidscd4$cd4.bl ^ lmd.bl - 1) / lmd.bl
 resar <- bcmmrm(outcome = cd4, group = treatment, data = aidscd4,
                time = weekc, id = id, covv = c("cd4.bl.tr", "sex"),
                cfactor = c(0, 1), structure = "AR(1)", glabel =
                c("Zid/Did", "Zid+Zal", "Zid+Did", "Zid+Did+Nev"))
 boxplot(resar, xlab = "Treatment", col = 1:4)
}
\seealso{
\code{\link{bcmmrm}}, \code{\link{bcmmrmObject}}
}
